%### CURAT.SL

% Code to use Cambridge University Lisp style rational numbers in
% Sheep, c.f. laritr.sl.

(SETGLOBALS IMNT!*)

(SAMEDEF SNUMP NUMBERP)

(SAMEDEF SATOM ATOM)

% X is a Snum. Test if it is a rational number.
(SAMEDEF SRATIONALP RATIONALP)

(NOTLOADTIME
 % X is a rational number, get numerator.
 (SAMEDEF NUMG NUMQ)

 % X is a rational number, get denominator.
 (SAMEDEF DENG DENQ)

 (SAMEDEF MKRATN RATIONAL)

 ) % End of NOTLOADTIME.

% X is Snum. Get numerator if rational, otherwise X.
(SAMEDEF SNUMQ NUMQ)

% X is Snum. Get denominator if rational, otherwise 1.
(SAMEDEF SDENQ DENQ)

% X is Snum. Get denominator if rational, otherwise NIL.
(DE SDENP (X) (COND ((NOT (AONEP (SETQ X (DENQ X)))) X)))

% Create normalized rational number from integer arguments.
(DE SNUM (N D) (COND
  ((OR (FLOATP N) (FLOATP D)) (QUOTIENT N D))
  (T (RATIONAL N D)) ))

(SAMEDEF MKRATQ RATIONAL)

(SAMEDEF SFLOATP FLOATP)

(SAMEDEF SFLOAT FLOAT)

(SAMEDEF SADD1 ADD1)

(SAMEDEF SSUB1 SUB1)

% Make 1/X.
(DE SRECIP (X) (COND
  ((FLOATP X) (RECIP X))
  (T (RATIONAL 1 X)) ))

% Add normalized numbers.
(SAMEDEF SPLUS PLUS)

% Multiply normalized numbers.
(SAMEDEF STIMES TIMES)

(SAMEDEF SPLUS2 PLUS2)

(SAMEDEF STIMES2 TIMES2)

% Subtract two normalized numbers.
(SAMEDEF SDIFFERENCE DIFFERENCE)

% Divide two normalized numbers.
(DE SQUOTIENT (N D) (COND
  ((OR (FLOATP N) (FLOATP D)) (QUOTIENT N D))
  (T (RATIONAL N D)) ))

% MINUSP for SNUM.
(SAMEDEF SMINUSP MINUSP)

% -SNUM
(SAMEDEF SMINUS MINUS)

% Absolute of SNUM.
(SAMEDEF SABS ABS)

% BAS^EX where BAS and EX are SNUMs and at least one of them is a
% floating point number.
% Improved version for systems where second argument to EXPT not
% have to be an integer. Added by JEA.
(SAMEDEF SFEXPT EXPT)

% BAS^EX where BAS is an SNUM and EX is an integer.
(SAMEDEF SEXPT EXPT)

% Integer part of X where X is an SNUM. Result is never greater than X.
(DE SFIX (X)
 (SLET (A)
  (COND
   ((SRATIONALP X)
    (SETQ A (QUOTIENT (NUMG X) (DENG X)))
    (COND
% This test (or redefinition of REMAINDER) necessary as REMAINDER may
% give negative results in Reduce based on culisp. JEA.
     ((MINUSP (REMAINDER (NUMG X) (DENG X))) (SUB1 A))
     (T A) ) )
   ((NOT (FLOATP X)) X)
   ((MINUSP X) (SUB1 (FIX X)))
   (T (FIX X)) ) ) )

% Fractional part of X where X is an SNUM. Result is never negative.
% As REMAINDER may give negative results in Reduce based on culisp
% use the general definition given in laritr.sl. JEA.
%(DE SFRACTION (X) (COND
%  ((SRATIONALP X) (RATIONAL (REMAINDER (NUMQ X) (DENQ X)) (DENQ X)))
%  ((NOT (FLOATP X)) 0)
%  ((MINUSP (SETQ X (DIFFERENCE X (FIX X))))    % For specific Lisp.
%   (ADD1 X) )
%  (T X) ))

(SAMEDEF SLESSP LESSP)

(SAMEDEF SGREATERP GREATERP)

% IMBAS rewritten to make use of SFIX rather than REMAINDER.
% This version is implementation independent. JEA.
(DE IMBAS (EX)                          % i^EX
 (COND
  ((OR (NOT (SNUMP EX)) (FLOATP EX))
   (XMKEXPT IMNT!* EX) )
  (T
   (SLET
    ((F (SFIX EX))
     (N (COND ((EVENP F) F) (T (SUB1 F))))
     (P (SDIFFERENCE EX N))
     (VL (COND
          ((AZEROP P) 1)
          (T (XMKEXPT IMNT!* P) ) ) ) )
    (COND
     ((EVENP (QUOTIENT N 2)) VL)
     ((NUMBERP VL) (SMINUS VL))
     (T (MKMIN VL)) ) ) ) ) )
%(DE IMBAS (EX) (COND                   % i^EX
%  ((NOT (SNUMP EX)) (XMKEXPT IMNT!* EX))
%  ((SRATIONALP EX)
%   (SLET ((DEN (DENQ EX)) (A (TIMES 2 DEN))
%          (VL (XMKEXPT IMNT!*
%               (RATIONAL (REMAINDER (SETQ EX (NUMQ EX)) A) DEN) )) )
%    (COND
%     ((EVENP (QUOTIENT EX A)) VL)
%     (T (MKMIN VL)) ) ) )
%  ((FLOATP EX) (XMKEXPT IMNT!* EX))
%  (T (XCASE (REMAINDER EX 4) NIL 1 IMNT!* -1 (SMMINUS IMNT!*))) ))

% SAMEDEF cannot be used because of bootstrap problems.
(DMACRO BIGGSNU (!&REST L) (CONS 'BIGGSN1 L))

(DMACRO BIGGSNRT (!&REST L) (CONS 'BIGGSN1 L))

(SAMEDEF COPYL COPY)

(SAMEDEF COPYA COPY)

% Defined in the current file.
% Note! The corresponding unflagging will now take place in shimp2.sl.
% JEA.
(FLAG '(SNUMP SATOM COPYL COPYA MKRATQ NUMG DENG MKRATN SRATIONALP
        SNUMQ SDENQ SDENP SNUM SFLOATP SFLOAT SADD1 SSUB1 SRECIP
        SPLUS STIMES SPLUS2 SDIFFERENCE SQUOTIENT STIMES2 SMINUSP
        SMINUS SABS SFEXPT SEXPT SFIX SLESSP SGREATERP
% SFRACTION  Removed. JEA.
        IMBAS BIGGSNU BIGGSNRT)
      'LOSE)

% Not needed for Culisp.
(FLAG '(RPLANUM RPLADEN CHKGCD NORM1 SPL2 ARITHP1 STI2 ARITHT1)
      'LOSE)

%### CURAT.SL end
