%### GETEN.SL

% This file contains the routines that access tensor-elements

(SETGLOBALS LIX!*)

(DE IGETEN (TN IL) 
% TN: id (tensor name), IL: list of indices
% Get tensor component if it is there, otherwise return NIL
%(SETQ TN (TENALIAS TN)) Already checked for. JEA.
 (COND 
  ((NULL IL) (SETQ LIX!* NIL) (IDVALUE TN)) % Scalar, get value if any.
  ((OR  % Check first if GETBLO returns zero for the case of tensor zero
    (NULL (SETQ TN (GETBLO TN))) % because of symmetry. Bugfix. JEA.
    (NULL (SETQ LIX!* (TENFIND IL (CAR TN)))) ) % If
   0)                   % vector index is NIL then 0 by symmetry.
  (T (TNGET (CDR TN))) ) )              % Get component or NIL

(DE TNGET (TNA) (SLET ((VX (GETV TNA LIX!*))) (COND
   ((NOT (EQCAR VX 'TNGET)) VX)
   (T (SETQ LIX!* (CADDR VX)) (XCASE (CADR VX) NIL
       (PROG1 LIX!* (SETQ LIX!* NIL))   % Stored constant
       0                                % 0 by symmetry
       (TNGET TNA)                      % symmetric
       (PROGN (SETQ MINSW (NOT MINSW)) (TNGET TNA)) % antisymmetric
       (PROGN                           % hermitian symmetry
        (SETQ CONJSW (NOT CONJSW))
        (TNGET TNA) ) )) )))

(DE EGETEN (TN IL TPL)
% TN: Tensor name, IL: index list, TPL: NIL or Tps-order (fixnum)
% Get tensor element (of Tps-order TPL if TPL is non-NIL). If element
% is found, return it, otherwise if 
 (SLET (A B C)
% (SETQ TN (TENALIAS TN))   % Use the alias if any. Trying without. JEA.
  (SETQ A (IGETEN TN IL))
  (SETQ B LIX!*)
  (COND
   ((NOT (AND A TPSSW)) NIL)
% Only interesting if have component and doing TPS.
   (TPL (SETQ A (OR             % Want order TPL.
      (TPGET A TPL)             % Get it if it's there, otherwise set A
      (PROGN (SETQ C (TPHIGHORD A)) NIL) )))    % to NIL and C to order.
   ((EQCAR A 'TPS)
% Mark which tensor component it was so that needed orders can
    (RPLACA (CDR A) (COND (IL (CONS TN B)) (T TN)))) ) % be made later.
  (COND
   (A A)                                % Found component.
   (NOSEQUENCE                          % Compute not found component.
    (SETQ B (COND (IL (CONS TN B)) (T TN)))     % Element pointer.
    (PLET ((MINSW MINSW) (CONJSW CONJSW) (HDIAG!* HDIAG!*))
% Preserve switches that where set by TENFIND.
     (COND                              % Make wanted orders.
      (C (FORL TPSSW ((XADD1 C) TPL) (XIMAKE B NIL)))
      (T (PLET ((TPSSW TPSSW)) (XIMAKE B NIL))) ) % Make component.
     (SLET (NOSEQUENCE)         % With NOSEQUENCE off to avoid looping,
      (EGETEN TN IL TPL) ) ) )          % get element or error.
   (T (ERRORFT 549  "Can't find element %r" (CONS TN IL))) ) ) )

(DE XGETEN (TN IL CONJSW)
% TN: tensor name, IL: index list, CONJSW (fluid): conjugation switch.
% Get a component. To be used in products.
% Doesn't copy. Handles conjugation but not minus. Picks whole TPS.
 (SLET (HDIAG!*)
  (SETQ TN (EGETEN TN IL NIL))
  (COND
   ((AND CONJSW (NOT HDIAG!*)) (CONJOF TN))
% Diagonal components are real.
   (T TN) ) ) )

(DE ZGETEN (TN IL CJSW MINSW) (PROGN
% Same as XGETEN, but also handles minus.
  (SETQ TN (XGETEN TN IL CJSW))
  (COND
   (MINSW (NSMMINUS TN))
   (T TN) ) ) )

(DE GETSCL (SCL) (EGETEN SCL NIL NIL))
% Fetch value of scalar SCL.

(DE GETEN (TN IL)
% This is the default routine to get tensor components.
 (ZGETEN TN IL NIL NIL) )

(DE CZGETEN (TN IL CJSW MINSW)
% TN: tensor name, IL: index list, CJSW: conjugation switch (not fluid),
% MINSW (fluid): minus switch.
% Get a component. To be used in sums.
% Copys. Handles conjugation and minus. Picks wanted TPS-order.
 (SETQ TN (CXGETEN TN IL CJSW)) % CJSW not fluid, CXGETEN binds CONJSW.
 (COND
  (MINSW (SMMINUS TN))
  (T TN) ) )

(DE CXGETEN (TN IL CONJSW)
% Same as CZGETEN, but doesn't handle minus. CONJSW is fluid.
 (SLET (HDIAG!*)
  (SETQ TN (COPYL (EGETEN TN IL TPSSW)))
  (COND
   ((AND CONJSW (NOT HDIAG!*)) (SETQ TN (CPLXCONJ TN)))
   (T TN) ) ) )

(DE PUTEN (TN IL SHEXPR) % Set tensor component TN(IL) to SHEXPR.
 (SETQ TN (TENALIAS TN))
 (COND
  ((NULL IL)                    % No indices given.
   (COND
    ((AND (TENP TN) (GETBLO TN))
     (RANKERR) )                % Tensor indices missing. Bugfix. JEA.
    (T (SET TN SHEXPR)) ) )     % Scalar, just SET it.
  (T (SLET (MINSW CONJSW HDIAG!* (A (GETBLO TN)) B)
      (COND ((NULL (SETQ B (TENFIND IL (CAR A))))
% 0 by symmetry, can't be set.
             (ERRORFT 547
              "Element %r is 0 by symmetry and can't be changed"
              (CONS TN IL) ) ))
      (COND (MINSW (SETQ SHEXPR (SMMINUS SHEXPR))))
      (COND
       ((AND CONJSW (NOT HDIAG!*)) (SETQ SHEXPR (CPLXCONJ SHEXPR))) )
      (PUTV (CDR A) B SHEXPR) )) ) )

(DE SETEN (TN SHEXPR)
% Set tensor component TN to copy of SHEXPR. Return SHEXPR.
 (SLET ((A NIL))
  (COND ((PAIRP TN) (SETQ A (CDR TN)) (SETQ TN (CAR TN))))
  (PUTEN TN A (COPYL SHEXPR))
  SHEXPR) )

% TINPUT restores a tensor component in save format.
(DE TINPUT (TNX SXP) (SLET (IL)    
  (COND ((PAIRP TNX) (SETQ IL (CDR TNX)) (SETQ TNX (CAR TNX))))
  (PUTEN TNX IL (SIMP1 (CEXIFORM SXP))) ))

%### GETEN.SL end
