%### LARITR.SL

% laritr.sl is used to handle rational arithmetics in Sheep where
% real rational numbers a/b are represented as (a . b) . Complex
% numbers are represented using I as imaginary unit where I^2=-1
% etc. is know. Lisp systems with built in handling of rational
% numbers (Cambridge Lisp) will not use all parts of this code. JEA.

% SNUMP tests if X is a Sheep number. Note that in order to to gain
% speed the test is incomplete, it assumes that X is valid Sheep
% expression.
(DE SNUMP (X) (OR (NUMBERP X) (AND (PAIRP X) (NUMBERP (CAR X)))))

(DE SATOM (X) (OR (ATOM X) (NUMBERP (CAR X))))

% X is a Snum. Test if it is a rational number.
(SAMEDEF SRATIONALP PAIRP)

(NOTLOADTIME
 % X and Y are normalized, i.e. X and Y are integers, GCD(X,Y)=1, Y > 1.
 % Make rational number.
 (SAMEDEF MKRATN CONS)

 % X is a rational number, get numerator.
 (SAMEDEF NUMG CAR)

 % X is a rational number, get denominator.
 (SAMEDEF DENG CDR)

 % Replace numerator of rational number X with integer Y.
 % No checking is done that Y and denominator of X are normalized.
 (SAMEDEF RPLANUM RPLACA)

 % Replace denominator of rational number X with integer Y.
 % No checking is done that numerator of X and Y are normalized.
 (SAMEDEF RPLADEN RPLACD)

 % Remove common factor from XX and YY.
 % Return common factor if not 1, else NIL.
 % XX and YY must be ids whos value might be changed.
 (DS CHKGCD (XX YY)
  (SLET ((C (GCD XX YY)))
   (COND ((NOT (AONEP C))
          (SETQ XX (QUOTIENT XX C)) (SETQ YY (QUOTIENT YY C)) C)) ) )
 ) % End of NOTLOADTIME.

% X is Snum. Get numerator if rational, otherwise X.
(DE SNUMQ (X) (COND
  ((SRATIONALP X) (NUMG X))
  (T X) ))

% X is Snum. Get denominator if rational, otherwise 1.
(DE SDENQ (X) (COND
  ((NUMBERP X) 1)
  ((AND (SRATIONALP X) (NUMBERP (CDR X))) (DENG X))
  (T (ERRORFT 491 "%r Not Sheep number" X)) ))

% X is Snum. Get denominator if rational, otherwise NIL.
(DE SDENP (X) (COND ((SRATIONALP X) (DENG X))))

% Create normalized rational number from integer arguments.
(DE SNUM (N D) (COND
  ((OR (FLOATP N) (FLOATP D)) (QUOTIENT N D))
  ((AZEROP D) (SRECIP D))               % Give error.
  ((MINUSP D) (NORM1 (MINUS N) (MINUS D)))
  (T (NORM1 N D)) ))

% Normalise, D > 0
(DE NORM1 (N D) (COND
  ((AZEROP N) 0)
  (T (CHKGCD N D) (MKRATQ N D)) ))

% N and D are integers. D > 0. GCD(N,D)=1. Make integer or rational.
(DE MKRATQ (N D) (COND
  ((AONEP D) N)         % Integer
  (T (MKRATN N D)) ))

(DE SFLOATP (N) (AND (NOT (SRATIONALP N)) (FLOATP N)))

(DE SFLOAT (N) (COND
  ((SRATIONALP N) (QUOTIENT (FLOAT (NUMG N)) (DENG N)))
  (T (FLOAT N)) ))

(DE SADD1 (X) (COND                     % X+1
  ((SRATIONALP X) (MKRATN (PLUS (NUMG X) (DENG X)) (DENG X)))
  (T (ADD1 X)) ))

(DE SSUB1 (X) (COND                     % X-1
  ((SRATIONALP X) (MKRATN (DIFFERENCE (NUMG X) (DENG X)) (DENG X)))
  (T (SUB1 X)) ))

% Make 1/X.
(DE SRECIP (X) (COND
  ((ZEROP X) (ERRORFT 490 "Zero denominator"))
  ((NOT (SRATIONALP X)) (COND
    ((FLOATP X) (QUOTIENT 1 X))
    ((MINUSP X) (MKRATQ -1 (MINUS X)))
    (T (MKRATQ 1 X)) ))
  ((MINUSP (NUMG X)) (MKRATQ (MINUS (DENG X)) (MINUS (NUMG X))))
  (T (MKRATQ (DENG X) (NUMG X))) ))

% Add normalized numbers.
(DMACRO SPLUS (!&REST U) (EXPAND U 'SPLUS2))

% Multiply normalized numbers.
(DMACRO STIMES (!&REST U) (EXPAND U 'STIMES2))

% Add two normalized numbers.
(DE SPLUS2 (X Y) (COND
  ((AZEROP X) Y)
  ((AZEROP Y) X)
  ((NOT (SRATIONALP X)) (COND
    ((SRATIONALP Y) (ARITHP1 X (NUMG Y) (DENG Y)))
    (T (PLUS X Y)) ))
  ((NOT (SRATIONALP Y)) (ARITHP1 Y (NUMG X) (DENG X)))
  ((EQN (DENG X) (DENG Y)) (NORM1 (PLUS (NUMG X) (NUMG Y)) (DENG X)))
  (T (SPL2 (NUMG X) (DENG X) (NUMG Y) (DENG Y))) ))

% Subtract two normalized numbers.
(DE SDIFFERENCE (X Y) (COND
  ((AZEROP X) Y)
  ((AZEROP Y) X)
  ((NOT (SRATIONALP X)) (COND
    ((SRATIONALP Y) (ARITHP1 X (MINUS (NUMG Y)) (DENG Y)))
    (T (DIFFERENCE X Y)) ))
  ((NOT (SRATIONALP Y)) (ARITHP1 (MINUS Y) (NUMG X) (DENG X)))
  ((EQN (DENG X) (DENG Y))
   (NORM1 (DIFFERENCE (NUMG X) (NUMG Y)) (DENG X)) )
  (T (SPL2 (NUMG X) (DENG X) (MINUS (NUMG Y)) (DENG Y))) ))

% Rational + rational. Denominators different.
(DE SPL2 (NX DX NY DY)
 (SLET ((Y DY) (X (CHKGCD DX DY)))
  (SETQ NX (PLUS (TIMES NX DY) (TIMES NY DX)))
  (COND ((AND X (CHKGCD NX X)) (SETQ Y (TIMES X DY))))
  (MKRATN NX (TIMES DX Y)) ) )

% (Integer or float) + rational
(DE ARITHP1 (X NY DY) (COND
  ((FLOATP X) (PLUS (QUOTIENT NY (FLOAT DY)) X))
  (T (MKRATN (PLUS (TIMES X DY) NY) DY)) ))

% Divide two normalized numbers.
(DE SQUOTIENT (N D) (COND
  ((NOT (SRATIONALP D)) (COND
    ((NOT (SRATIONALP N)) (SNUM N D))
    ((SRATIONALP (SETQ D (SNUM (NUMG N) D)))
     (RPLADEN D (TIMES (DENG N) (DENG D))))
    ((FLOATP D) (QUOTIENT D (DENG N)))
    (T (MKRATN D (DENG N))) ))
  ((NOT (SRATIONALP N)) (COND
    ((ZEROP N) N)
    ((SRATIONALP (SETQ N (SNUM N (NUMG D))))
     (RPLANUM N (TIMES (DENG D) (NUMG N))))
    (T (TIMES N (DENG D))) ))   % lh added 2 rparens.
  ((AONEP (NUMG D)) (ARITHT1 (DENG D) (NUMG N) (DENG N)))
  ((AEQN (NUMG D) -1) (ARITHT1 (MINUS (DENG D)) (NUMG N) (DENG N)))
  ((MINUSP (NUMG D))
   (STI2 (NUMG N) (DENG N) (MINUS (DENG D)) (MINUS (NUMG D))))
  (T (STI2 (NUMG N) (DENG N) (DENG D) (NUMG D))) ))  % lh removed 2 rpar

% Multiply two normalized numbers.
(DE STIMES2 (X Y) (COND
  ((OR (AZEROP X) (AZEROP Y)) 0)
  ((AONEP X) Y)
  ((AONEP Y) X)
  ((NOT (SRATIONALP X)) (COND
    ((SRATIONALP Y) (ARITHT1 X (NUMG Y) (DENG Y)))
    (T (TIMES X Y)) ))
  ((NOT (SRATIONALP Y)) (ARITHT1 Y (NUMG X) (DENG X)))
  ((EQN (DENG X) (DENG Y))
   (MKRATN (TIMES (NUMG X) (NUMG Y)) (TIMES (DENG X) (DENG Y))) )
  (T (STI2 (NUMG X) (NUMG Y) (DENG X) (DENG Y))) ))

% Rational * rational
(DE STI2 (NX NY DX DY)
 (CHKGCD NX DY)
 (CHKGCD NY DX)
 (MKRATQ (TIMES NX NY) (TIMES DX DY)) )

% (Integer or float) * rational
(DE ARITHT1 (X NY DY) (COND
  ((FLOATP X) (TIMES (QUOTIENT NY (FLOAT DY)) X))                      
  (T (CHKGCD X DY) (MKRATQ (TIMES X NY) DY)) ))

% MINUSP for SNUM
(DS SMINUSP (SNU) (MINUSP (SNUMQ SNU)))

% -SNUM
(DE SMINUS (SNU) (COND
 ((SRATIONALP SNU) (MKRATN (MINUS (NUMG SNU)) (DENG SNU)))
 (T (MINUS SNU)) ))

% Absolute of SNUM.
(DE SABS (SNU) (COND
 ((NOT (SRATIONALP SNU)) (ABS SNU))
 ((MINUSP (NUMG SNU)) (MKRATN (MINUS (NUMG SNU)) (DENG SNU)))
 (T SNU) ))

% BAS^EX where BAS and EX are SNUMs and at least one of them is a
% floating point number.
% Can be improved for systems where second argument to EXPT not
% have to be an integer or have EXP and LOG defined. Added by JEA.
(DE SFEXPT (BAS EX) (COND
  ((FIXP EX) (EXPT BAS EX))     % EX integer so BAS must be a float.
  (T (XMKEXPT BAS EX)) ))

% BAS^EX where BAS is an SNUM and EX is an integer.
(DE SEXPT (BAS EX)
 (SLET ((SW (MINUSP EX)) (DEN (SDENP BAS) ))
  (COND (SW (SETQ EX (MINUS EX))))
  (SETQ BAS
   (EXPT (COND (DEN (SETQ DEN (EXPT DEN EX)) (NUMG BAS)) (T BAS)) EX) )
  (COND
   ((NULL SW) (COND
     (DEN (MKRATN BAS DEN))
     (T BAS) ))
   ((AZEROP BAS) (SNUM 1 0))
   ((MINUSP BAS) (MKRATQ (COND (DEN (MINUS DEN)) (T -1)) (MINUS BAS)))
   (T (MKRATQ (COND (DEN DEN) (T 1)) BAS)) ) ) )

% Integer part of X where X is an SNUM. Result is never greater than X.
(DE SFIX (X)
 (SLET (A)
  (COND
   ((SRATIONALP X)
    (SETQ A (QUOTIENT (NUMG X) (DENG X)))
    (COND
     ((MINUSP (REMAINDER (NUMG X) (DENG X))) (SUB1 A)) % Improve
     (T A) ) )
   ((NOT (FLOATP X)) X)
   ((GREATERP (SETQ A (FIX X)) X) (SUB1 A))     % For specific Lisp.
   (T A) )))

% Fractional part of X where X is an SNUM. Result is never negative.
(DE SFRACTION (X)
 (SLET (A)
  (COND
   ((SRATIONALP X)
    (COND
     ((MINUSP (SETQ X (REMAINDER (NUMG X) (SETQ A (DENG X))))) % Improve
      (SETQ X (PLUS X A)) ))
    (MKRATN X A) )
   ((NOT (FLOATP X)) 0)
   ((MINUSP (SETQ X (DIFFERENCE X (FIX X))))
    (ADD1 X) )          % for specific LISP.
   (T X) )))

(DE SLESSP (X Y) (COND
  ((SRATIONALP X) (COND
    ((SRATIONALP Y)
     (LESSP (TIMES (NUMG X) (DENG Y)) (TIMES (NUMG Y) (DENG X))) )
    (T (LESSP (NUMG X) (TIMES Y (DENG X)))) ))
  ((SRATIONALP Y) (LESSP (TIMES X (DENG Y)) (NUMG Y)))
  (T (LESSP X Y)) ))

(DE SGREATERP (X Y) (COND
  ((SRATIONALP X) (COND
    ((SRATIONALP Y)
     (GREATERP (TIMES (NUMG X) (DENG Y)) (TIMES (NUMG Y) (DENG X))) )
    (T (GREATERP (NUMG X) (TIMES Y (DENG X)))) ))
  ((SRATIONALP Y) (GREATERP (TIMES X (DENG Y)) (NUMG Y)))
  (T (GREATERP X Y)) ))

(DE MIN1BAS (EX) (COND                  % -1^EX
  ((NOT (SNUMP EX)) (XMKEXPT -1 EX))    % exponent not a number
  ((SRATIONALP EX) (SLET ((DEN (DENG EX)))
    (SETQ EX (NUMG EX))
    (COND
     ((EVENP DEN) (IMBAS (MKRATQ EX (QUOTIENT DEN 2))))
     ((EVENP EX) 1)
     (T -1) ) ))
  ((FLOATP EX) (XMKEXPT -1 EX))
  ((EVENP EX) 1)
  (T -1) ) )

(DE IMBAS (EX) (SLET (DEN A SW) (COND   % i^EX
   ((NOT (SNUMP EX)) (XMKEXPT IMNT!* EX))
   ((SRATIONALP EX)
    (SETQ A (TIMES 2 (SETQ DEN (DENG EX))))
    (SETQ SW (EVENP (QUOTIENT (SETQ EX (NUMG EX)) A)))
    (COND ((MINUSP (SETQ EX (REMAINDER EX A)))
           (SETQ EX (PLUS EX A)) (SETQ SW (NOT SW)) ))
    (SETQ EX (XMKEXPT IMNT!* (MKRATN EX DEN)))
    (COND
     (SW EX)
     (T (MKMIN EX)) ) )
   ((FLOATP EX) (XMKEXPT IMNT!* EX))
   (T (SETQ EX (REMAINDER EX 4))
      (COND ((IMINUSP EX) (SETQ EX (IPLUS EX 4))))
      (SETQ A (COND
        ((IEVENP EX) (SETQ !*REORDSW T) 1)
        (T IMNT!*) ))
      (COND
       ((ILESSP EX 2) A)
       (T (SMMINUS A)) ) ) )))

(DE LPLUS (L) (SLET ((A 0))
  (FOREACH X IN L DO (SETQ A (SPLUS X A)))
  A))

(DE LTIMES (L) (SLET ((A 1))
  (FOREACH X IN L DO (SETQ A (STIMES X A)))
  A))

(DE VNUM (SNU) (SLET (RT)
  (SETQ SNU (COND
    ((SRATIONALP SNU)
     (SETQ RT RATPAR)
     (COND
      ((MINUSP (NUMG SNU))
       (CONSA '!- (MINUS (NUMG SNU)) '!/ (DENG SNU)) )
      (T (CONSA (NUMG SNU) '!/ (DENG SNU))) ) )
    ((MINUSP SNU) (CONS '!- (MINUS SNU)))
    (T SNU) ))
  (COND
   (RT (SETPARSW TIMPAR DIFPAR BASPAR))
   (T (SETPARSW DIFPAR BASPAR)) )
  SNU))

(DE BIGGSNU (X Y) (BIGGSNRT X Y))

(COMMENT (COND
  ((NOT NORATORDN) (BIGGSNRT X Y))
  ((NOT (SRATIONALP X))
   (COND ((SRATIONALP Y) 'INSERT) (T (BIGGSN1 X Y))) )
  ((SRATIONALP Y)
   (OR (BIGGSN1 (NUMG X) (NUMG Y)) (BIGGSN1 (DENG X) (DENG Y))) )
  (T 'GOON) ) )

(DE BIGGSNRT (X Y)
 (COND
  ((NOT (SRATIONALP X)) (COND
    ((SRATIONALP Y) (BIGGSN1 (TIMES X (DENG Y)) (NUMG Y)))
    (T (BIGGSN1 X Y)) ))
  ((NOT (SRATIONALP Y)) (BIGGSN1 (NUMG X) (TIMES Y (DENG X))))
  ((EQN (DENG X) (DENG Y)) (BIGGSN1 (NUMG X) (NUMG Y)))
  (T (BIGGSN1 (TIMES (NUMG X) (DENG Y)) (TIMES (NUMG Y) (DENG X)))) ) )

(DE CIEXFORM (SXP) (COND
  ((NOT (SATOM SXP))
   (CONS (CAR SXP) (FOREACH X IN (CDR SXP) COLLECT (CIEXFORM X))) )
  ((SRATIONALP SXP) (LIST 'QUOT (NUMG SXP) (DENG SXP)))
  (T SXP) ))

(DE QUOT (NUM DEN) (QUOTIENT (FLOAT NUM) DEN))

(DE CEXIFORM (SXP) (COND
  ((EQCAR SXP 'QUOT) (MKRATN (CADR SXP) (CADDR SXP)))                
  ((SATOM SXP) SXP)
  (T (FOREACH X IN (CDR SXP) REPLACE (CEXIFORM X)) SXP) ))

(SETQ IMNT!* 'I)

(COMPILETIME
 (FOREACH X IN '(MKRATN DENG NUMG RPLADEN RPLANUM CHKGCD) DO
  (REMOB X) ) )

%### LARITR.SL end
