%### PRIFAC.SL

% Factorisation of integers.
% Written by Jim Skea (QMC, London) at Stockholms Universitet.
% Large changes by Inge Frick.

% PRIMEFACTORL factors an integer into its prime factors.
% Negative integers return -1 and the prime factorisation of ABS(INT).

% Rather slow for nearly-prime integers > (2^23)-1.
% Not considered important as Mersenne numbers this large are seldom
% encountered in everyday General Relativity.

(DE MERSENNE (N) (SUB1 (EXPT 2 N))) % For testing, but quite fun.

(SETGLOBALS
 (SMALLPRIMELIST!*
  (3 5 7 11 13 17 19 23 29 31 37 41 43 47 53 59 61 67 71 73 79
   83 89 97 101 103 107 109 113 127 131 137 139 149 151 157 163) )
 ((SMALLPRIMESQR!* (FOREACH X IN SMALLPRIMELIST!* COLLECT (EXPT X 2))))
 ((PRIMELIST!*
   (LIST 167 173 179 181 191 193 197 199 211 223 227 229 233 239 241
         251 257 263 269 271 277 281 283 293 307 311 313 317 331 337
         347 349 353 359 367 373 379 383 389 397 401 409 419 421 431
         433 439 443 449 457 461 463 467 479 487 491 499 503 509 521
         523 541 547 557 563 569 571 577 587 593 599 601) ))
 ((SMALLPRIMELIM!* (ADD1 (CAR (LASTPAIR SMALLPRIMELIST!*)))))
% This must be less than the square of SMALLPRIMELIM!*
 (PRIMELISTLIM!* 6240) )

(NEWSWITCHES
 (NOFACTORMSG
  "(EXPERIMENTAL) Do not print message when only large factors" ) )

(DE PRIMEFACTORS (INT) (COND
  ((NOT (MINUSP INT)) (PPRIMEFACTORS INT))
  ((PAIRP (SETQ INT (PPRIMEFACTORS (MINUS INT))))     % Bugfix. JEA.
   (CONS -1 INT) )
  ((AONEP INT) -1)
  (T (LIST -1 INT)) ))

(DE PPRIMEFACTORS (INT) (PROG (CNT FL PL D SSW)
  (COND
   ((AMEMQ INT '(2 1 0)) (RETURN INT))
   ((AND (LESSP INT SMALLPRIMELIM!*) (AMEMQ INT SMALLPRIMELIST!*))
    (RETURN INT) ) )     % INT is often small prime
  (SETQ CNT 0)
  (WHILE (EVENP INT) (INCR CNT) (SETQ INT (QUOTIENT INT 2)))
  (COND
   ((AONEP CNT) (XPUSH FL 2))
   ((NOT (AZEROP CNT)) (XPUSH FL (CONS 2 CNT))) )
  (SETQ PL SMALLPRIMELIST!*)
  (UNTIL (NOT (AND PL (GREATERP (CAR D) (CAR PL))))
   (SETQ CNT 0)
   (WHILE (AZEROP (CDR (SETQ D (DIVIDE INT (CAR PL)))))
          (SETQ INT (CAR D)) (INCR CNT))
   (COND
    ((AONEP CNT) (XPUSH FL (CAR PL)))
    ((NOT (AZEROP CNT)) (XPUSH FL (CONS (CAR PL) CNT))) )
   (SETQ PL (COND
     ((CDR PL) (CDR PL))
     ((NOT SSW) (SETQ SSW T) PRIMELIST!*)
     ((LESSP (CAR PL) PRIMELISTLIM!*)
      (CDR (RPLACD PL (NEXTPRIME (CAR PL)))) )
     ((NOT NOFACTORMSG)
      (WARNINGFT 148
       "%p has no factors less than or equal %p" INT (CAR PL) )
      NIL) )) )
  (COND ((NOT (AONEP INT)) (XPUSH FL INT)))
  (RETURN (COND
    ((CDR FL) (REVERSIP FL))
    ((PAIRP (CAR FL)) FL)
    (T (CAR FL)) )) ))

(DE NEXTPRIME (TRY) (SLET (LM PL PLS)
  (UNTIL PL
   (SETQ TRY (PLUS TRY 2))
   (SETQ PLS SMALLPRIMESQR!*)
   (SETQ PL SMALLPRIMELIST!*)
   (WHILE (AND (UNPUSH PLS) (LESSP (CAR PLS) TRY)) (UNPUSH PL))
   (SETQ LM (CAR PL))
   (SETQ PL SMALLPRIMELIST!*)
   (WHILE (AND PL (NOT (GREATERP (CAR PL) LM)))
    (SETQ PL (COND
      ((AZEROP (REMAINDER TRY (CAR PL))) NIL)
      (T (CDR PL)) )) ) )
  (NCONS TRY) ))

%### PRIFAC.SL end
