%### TEXPD.SL

% This file defines the functions (T)EXPD(X) and EXTI that implement
% the distributive and associative laws.

(DE EXPD (SXP) (EXPDX SXP 'PLUS 'TIMES))

(DE TEXPD (SXP) (TEXPDX SXP 'PLUS 'TIMES))

(COMMENT        % Start of commented out code.
(PUT 'TIMES 'TIMESPLUSOP 'PLUS)

(FLAG '(PLUS) 'PLUSOP)

(DE VEXPD (SXP)
 (SLET (A)
  (COND
   ((SATOM SXP) SXP)
   ((FLAGP (SETQ A (CAR SXP)) 'PLUSOP)
    (FOREACH X IN (CDR SXP) REPLACE (VEXPD X))
    (EXTI SXP) )
   ((SETQ A (GET A 'TIMESPLUSOP))
    (FOREACH X IN (CDR SXP) REPLACE (VEXPD X))
    (EXTI (TEXPDX SXP A (CAR SXP))) )
   (T SXP) ) ) )
)       % End of commented out code.

(DE EXPDX (SXP PL TI)
% Applies distributive and associative laws recursively on expression
% SXP using PL as sum operator and TI as product operator.
% Order between factors is preserved.
% Order of terms is taken from order in factors that are sums.
 (COND ((OR (EQCAR SXP PL) (EQCAR SXP TI)) % If it is a sum or a
        (FOREACH X IN (CDR SXP) REPLACE % product, then apply EXPDX
         (EXPDX X PL TI) )              % on all terms or factors.
        (COND                           % If it is a product that
         ((AND (EQ (CAR SXP) TI) (MEMQCAR (CDR SXP) PL)) % contains
          (ITEXPDX SXP PL)              % sums, then expand and on each
          (FOREACH X IN (CDR SXP) REPLACE % term in the resulting sum
           (EXTI (CONS TI (COPYA X))) ) ) % apply the associative law,
         (T (EXTI SXP)) ) ))    % else just apply associative law.
 SXP)                           % Return the changed expression.

(DE EXTI (SXP)                          % Associative law.
% Applies the associative law to splice into the top level list those
% expressions that start with the same operator as the top level. This
% is not recursive, only elements of the top level are processed.
 (SLET ((MSK (CAR SXP))         % Remember "sum" operator in MSK.
        (A SXP) (B (CDR SXP)))% A is before first term, B at first term.
  (UNTIL (NULL B)                       % Until no more terms:
   (COND
    ((EQCAR (CAR B) MSK)                % If term is a "sum", then
     (UNPUSH B)                 % (B is now at next term) put term
     (RPLACD A (CDADR A))       % (operator removed) into top "sum" and
     (RPLACD (SETQ A (LASTPAIR A)) B) ) % put next term at end of term.
    (T (SETQ B (CDR (SETQ A B)))) ) )   % Get next term.
  SXP) )

(DE TEXPDX (SXP PL TI)                  % Distributive law
% Applies the distributive law to convert a product of sums into a sum
% of products.
 (AND
  (EQCAR SXP TI)                        % If it is a product
  (MEMQCAR (CDR SXP) PL)                % that contains sums,
  (FOREACH X IN (CDR (ITEXPDX SXP PL)) % then expand and
   REPLACE (CONS TI (COPYA X)) ) )      % copy result.
 SXP)

(DE ITEXPDX (SXP PL)            % Expand product that contains sums.
 (SLET ((A (REVERSIP (CDR SXP))) XSUM XSUM1)
  (RPLACD SXP (LIST NIL))               % Will become the sum.
  (FOREACH X IN A DO (COND              % X is a factor in the product.
    ((NOT (EQCAR X PL))         % If X not is a sum, then multiply
     (FOREACH Y IN (CDR SXP) REPLACE (CONS X Y)) ) % each term with X.
    (T (SETQ XSUM1 (CDR (SETQ XSUM SXP))) % The previous resulting sum.
       (FOREACH Y IN (CDR X) DO % Each term Y in X is multiplied with
        (FOREACH Z IN XSUM1 DO  % each term Z in the previous result
         (RPLACD XSUM (SETQ A (LIST (CONS Y Z)))) % to form a new term
         (SETQ XSUM A) ) ) ) ))         % in the result.
  (RPLACA SXP PL) ) )           % Put a summation operator in front.

(DE MEMQCAR (L MSK)
% Return true if any of the elements of L is a list beginning with MSK.
 (PROGN (WHILE (AND L (NOT (EQCAR (CAR L) MSK))) (UNPUSH L)) L) )

%### TEXPD.SL end
