%### EXFN2.SL =======================================================
%
%       EXFN2.SL
%       --------
%
%       Copyright (c) 1982 by Inge Frick, Institute of Theoretical
%       Physics, Vanadisvagen 9, S-113 46 Stockholm, Sweden.
%
%       This file contains some changes to EXFN2I.SL that adapts it to
%       Franz Lisp.
%
%====================================================================

(declare (special !*KEEPCASE))

% GLOBALs in EXFN2I.SL
(SETGLOBALS
 ((TAB!* (INTERN (ascii 9))))           % Tab. Added by JEA.
 TOPDIR!*
 (HOMEDIR!* (!~))
 ((LISPLIBDIR!* (LIST lisp!-library!-directory))) )

(SETFLUIDS LOADMSGTAB!*)

(COMMENT % THROWQ, CATCHQ and UNWINDPROTECT removed
(DS THROWQ (TAG FORM)
 (PROGN (SETQ THROWSIGNAL!* T) (!*throw (SETQ THROWTAG!* 'TAG) FORM)) )

(DS CATCHQ (TAG FORM) (!*catch 'TAG FORM))

(samedef 'UNWINDPROTECT 'unwind!-protect)
) % End of comment

(DS ERRSET (FORM SW) (PLET ((!*ERRMSG SW)) (errset FORM NIL)))

% PROG1([XPR:form]):any
% Evaluates the XPR from left to right and returns the value of the
% first XPR.
(samedef 'PROG1 'prog1)

(samedef 'EVENP 'evenp)

% The following arithmetic functions should be used when operating on
% integers that are sufficiently small (e.g. one word) that the
% operations can be open coded by a compiler. Numbers less than
% expt(30) are ok.

(mapc 'samedef
 '(XPLUS XTIMES XADD1 XSUB1 XDIFFERENCE XQUOTIENT XREMAINDER XMINUSP)
 '(!+    !*     !1!+  !1!-  !-          !/        !\         minusp ) )

(mapc 'samedef
 '(XEQN XLESSP XGREATERP)
 (IFP (getd '!=!&) '(!=!& !<!& !>!&) '(!= !< !>)) )

% The following functions with prefix XI operate on integers with
% absolute value < 64000. On some systems this means EQ can be used.

(mapc 'samedef
 '(XIPLUS XITIMES XIADD1 XISUB1 XIDIFFERENCE XIQUOTIENT XIREMAINDER
   XIMINUSP)
 '(!+     !*      !1!+   !1!-   !-           !/         !\
   minusp ) )

(mapc 'samedef
 '(XIEQN XILESSP XIGREATERP)
 (IFP (getd '!=!&)
 '(!=!&  !<!&    !>!&)
 '(!=    !<      !>)) )

% The following functions with prefix I and A operate on integers with
% absolute value < 1000. On many systems this means EQ can be used.
% Those with prefix A will also handle any argument but only guarantee
% to test equality when one argument is a small integer. Those with
% prefix I might give anything when an argument is not a small integer.
% Use I when both arguments are known to be small.
% Use A when one argument is known to be small.

(mapc 'samedef
 '(IPLUS ITIMES IADD1 ISUB1 IDIFFERENCE IQUOTIENT IREMAINDER IEQN
   IMINUSP AEQN AMEMQ AATSOC)
 '(!+    !*     !1!+  !1!-  !-          !/        !\         EQ
   minusp  EQ   memq  assq  ) )

(mapc 'samedef
 '(ILESSP IGREATERP)
 (IFP (getd '!=!&) '(!<!& !>!&) '(!< !>)) )

(samedef 'COPY 'copy)

(samedef 'DELQIP 'delq)

(IFP (getd 'rassq) (samedef 'RATSOC 'rassq))

(samedef 'LASTPAIR 'last)       % Returns last pair in list X.

(DS FLAG1 (X Y) (PUT X Y 'IsFlag))

(DS REMFLAG1 (X Y) (REMPROP X Y))

(samedef 'PLIST 'plist)

(DS FOREACHID (X &REST L) (FOREACH X IN (oblist) DO . L))

% GETFN(ARG:any):{NIL,codepointer,LAMBDA-expression}
% If ARG is a function then the definition is returned, else NIL.
(DE GETFN (X)
 (AND (IDP X)
  (SLET ((A (get X 'FNtyP))) (COND
    (A (CDR A))
    ((NULL (SETQ A (getd X))) NIL)
    ((EQ (COND ((PAIRP A) (CAR A)) (T (getdisc A))) 'LAMBDA) A)
    (T (CDR (GETD X))) )) ) )

% GETFNTYPE(ARG:any):{NIL,EXPR,FEXPR,MACRO}
% If ARG is a function, return the function type, else NIL.
(DE GETFNTYPE (A) (COND
  ((NULL (AND (IDP A) (SETQ A (getd A)))) NIL)
  ((EQ (SETQ A (COND ((PAIRP A) (CAR A)) (T (getdisc A)))) 'macro)
   'MACRO)
  ((EQ A 'nlambda) 'FEXPR)
  (T 'EXPR) ))

% FNP(ARG:any):extrabolean
% FNP returns NIL if ARG is not a function, else true.
(DE FNP (X) (AND (IDP X) (getd X)))

(DS EXPRP (X) (EQ (GETFNTYPE X) 'EXPR))

(DS FEXPRP (X) (EQ (GETFNTYPE X) 'FEXPR))

(DS MACROP (X) (EQ (GETFNTYPE X) 'MACRO))

(DMACRO ERRSETT (!&REST U) (LIST 'errset (MKPROGN U) nil))

(samedef 'FEXPRCALL 'funcall)

%(samedef 'DO 'do)

(DE INTERNP (X)
 (SLET ((Y (uconcat X)) (Z (intern Y)))
  (COND ((EQ Y Z) (REMOB Y)))
  (rematom Y)
  (COND
   ((EQ Z X) 'T) ) ) )

(DE ULCASE (X SW) (COND (SW (Ukey X)) (T (LKEY X))))

(samedef 'CONCATIDL 'concatl)

% MAKEID(CHARS:idlist,SW:any):id
% CHARS must be a list acceptable to COMPRESS. The value of MAKEID
% is an id made out of all characters in CHARS (including special
% characters).
% The id is INTERNed if SW is true, otherwise not.
(DE MAKEID (X Y)
 (COND
  (Y (NILutls (APPLY 'concat X)))
  (T (maknam X)) ) )

% MAKESTR(CHARS:charlist):string
% MAKESTR returns a string made from the character ids in CHARS.
(DE MAKESTR (X)
 (COND
  ((NULL X) (readlist '(!" !")))
  (T (SLET ((Y (LASTPAIR X)))
      (RPLACD Y '(!"))
      (SETQ X (readlist (CONS '!" X)))
      (RPLACD Y NIL)
      X)) ) )

% MAKEINT(DIGITS:digitlist,RADIX:{NIL,integer}):integer
% MAKEINT returns an integer made from the digits in DIGITS, with
% radix RADIX.
% Radix NIL is the same as decimal radix.
% If radix is an id other than NIL then it must be a function.
(DE MAKEINT (X Y)
 (SLET (A)
  (COND
   ((NULL X) 0)
   ((NULL Y) (readlist X))
   ((NUMBERP Y) (COND
     ((AND (NOT (XMINUSP Y)) (XLESSP Y 11))
      (SLET ((ibase Y)) (readlist X)) )
     (T (SETQ A (GET (CAR X) 'DIGITVALUE))
        (MAPC (CDR X) (FUNCTION (LAMBDA (B)
           (SETQ A (PLUS (TIMES A Y) (GET B 'DIGITVALUE))) )))
        A) ))
   ((SETQ A (GET Y 'MAKEINTFN)) (APPLY A (LIST X)))
   (T (ERROR 0 (LIST Y "is not a radix"))) ) ) )

% MAKEFLO(INTEGERPART:digitlist,FRACTIONPART:digitlist,
%         EXPONENT:digitlist):float
% MAKEFLO returns a floating point number made from the digits as
% <INTEGERPART>.<FRACTIONPART>E<EXPONENT>.
(DE MAKEFLO (X Y Z) (PROGN
  (COND (Z (SETQ Y (APPEND Y (CONS 'E Z)))))
  (readlist (APPEND X (CONS '!. (COND (Y) (T '(!0)))))) ))

(COMMENT %Cannot be used, bug? in marray.
(DE LIST2VECTOR (L)
 (SLET ((A (LENGTH L)))
  (COND
   ((XLESSP A 128) (makhunk L))
   (T
    (SETQ A
     (marray (small!-segment 'value A) (getd 'arrac!-oneD) 'Slisp A 4) )
    (fillarray A L) ) ) ) ) )

(declare
 (special Standard!-Output Standard!-Input poport piport
  PGnum!* LInum!* ))

(DE ORDERP (X Y) (!< (maknum X) (maknum Y)))

(samedef 'LEXP 'alphalessp)

(DS STDINP () (OR (NULL piport) (EQ piport Standard!-Input)))

(DS STDOUTP () (OR (NULL poport) (EQ poport Standard!-Output)))

(DE FILEP (X) (AND (OR (IDP X) (STRINGP X)) (probef X)))

(DE PRIN2D (X) (PRIN2 X) (drain) X)

(DE PRIN1D (X) (PRIN1 X) (drain) X)

%(samedef 'DRAIN 'drain)

(samedef 'TYO 'tyo)

(samedef 'TYIPEEK 'tyipeek)

(DM UNASCII (U) (LIST 'getcharn (CADR U) 1))

(IFP (getd 'untyi) (DE UNREADCH (X) (untyi (UNASCII X))))

(DE SKIPTO (X)
 (SETQ X (UNASCII X))
 (UNTIL (IEQN (TYI) X)) )

(IFP (getd 'tab) (samedef 'TTAB 'tab))

(DE BELL () (tyo 7))

(DE IOCHANS (CHI CHO BODY) (SLET (LB LA)
  (COND (CHI (XPUSH LB (CAR CHI)) (XPUSH LA 'piport)))
  (COND (CHO (XPUSH LB (CAR CHO)) (XPUSH LA 'poport)))
  (CONS (CONSA 'LAMBDA LA BODY) LB) ))

% FILEIN(FIL:file,[ARG:form]):any
% Evaluates ARGs with input switched to file FIL.
(DS FILEIN (FIL !&REST ARGS)
 (PLET ((INCH!$!$ (OPEN FIL 'INPUT)))
  (unwind!-protect 
   (SLET ((piport INCH!$!$) (LInum!* 1) (PGnum!* 1)) . ARGS)
   (CLOSE INCH!$!$) ) ) )

% FILEOUT(FIL:file,[ARG:form]):any
% Evaluates ARGs with output switched to file FIL.
(DS FILEOUT (FIL !&REST ARGS)
 (PLET ((OUTCH!$!$ (OPEN FIL 'OUTPUT)))
  (unwind!-protect
   (SLET ((poport OUTCH!$!$) (linel FILELINELENGTH!*)) . ARGS)
   (CLOSE OUTCH!$!$) ) ) )

(COMMENT
(DE SETCHARFOLD (SW) (PROG1 (status uctolc)
  (COND
   (SW (sstatus uctolc t))
   (T (sstatus uctolc NIL)) ) ))

(DS CHARFOLD (SW !&REST L) (SLET ((CHARF!&VL (SETCHARFOLD SW)))
  (unwind!-protect (PROGN . L) (SETCHARFOLD CHARF!&VL)) ))
)

% SETGCMESSAGE(SW:boolean):boolean
% Sets gcmessage status and returns old value. If gcmessage is true,
% then on most systems a message is printed when a garbage collection
% occurs.
(DE SETGCMESSAGE (SW) (PROG1 !*GC (SETQ !*GC SW)))

% GCMESSAGE(SW:boolean,[SXPR:form]):any
% Evaluate all SXPR while gcmessage status temporarily is set to SW.
(DS GCMESSAGE (SW !&REST L) (PLET ((!*GC SW)) . L))

(DE I!$PRNEVL (PRNEVL!$X PRESW!$ POSTV!$)
 (COND (PRESW!$ (TERPRI) (TERPRI)))
 (SETQ PRNEVL!$X (EVAL PRNEVL!$X))
 (COND ((NOT (AND POSTV!$ (EQ PRNEVL!$X POSTV!$)))
        (QTRPRI) (PRIN1 PRNEVL!$X) (TERPRI) ))
 PRNEVL!$X)

(samedef 'QUIT 'exit)

(samedef 'BYE 'exit)

% SHPDEBUG(SW:boolean):boolean
% If SW is true various debugging modes are turned on.
% What this function does is system dependent. On some systems SW can
% have more values than true and NIL.
% SHPDEBUG returns the old value of debugging.
(DE SHPDEBUG (X)
 (PROG1 !*SHPDEBUG
  (COND
   ((NULL X) (SETQ X 1))
   ((NOT (NUMBERP X)) (SETQ X 8)) )
  (SETQ !*SHPDEBUG X)
  (COND
   ((NULL (SETQ !*ERRMSG (NOT (AZEROP X))))
    (SETQ !*MSG (SETQ !*GC (SETQ !*LOADMSG NIL))) )
   (T (SETQ !*BAKGAG
       (SETQ !*BACKTRACE
        (XCASE X 0 NIL NIL 1 -1 4 -4 10 -10 T) ))
      (COND ((IGREATERP X 1)
             (SETQ !*MSG (SETQ !*GC (SETQ !*LOADMSG T))) ))
      (COND ((AND !*INT (IGREATERP X 9))
             (DEBUGGING T (IGREATERP X 11)))) ) ) ) )

(DE TIME () (FIX (TIMES 16.666666666666666 (CAR (ptime)))))

(DE GCTIME () (FIX (TIMES 16.666666666666666 (CADR (ptime)))))

(mapcar 'samedef
 '(BOX    UNBOX BOOLE LSH)
 '(maknum fake  boole lsh))

(mapcar 'samedef
 '(BIGP ASCII)
 '(bigp ascii))

(samedef 'RECLAIM 'gc)

(DM TRA (X) (RPLACA X 'trace))

(DM UNTRA (X) (RPLACA X 'untrace))

(eval!-when (eval compile)
 (FLAG '(WARNING SKIPWHITE PGLINE EXPLODEC RESET)
       'LOSE)  % These have been defined in SLFUNS.L.
 (FLAG '(EXPLODECHELP) 'LOSE) % Not needed.
 (FLAG '(EVENP XPLUS XTIMES XADD1 XSUB1 XDIFFERENCE XQUOTIENT
         XREMAINDER XEQN XLESSP XGREATERP XMINUSP AMEMQ AATSOC AEQN
         XIPLUS XITIMES XIADD1 XISUB1 XIDIFFERENCE XIQUOTIENT
         XIREMAINDER XIEQN XILESSP XIGREATERP XIMINUSP
         IPLUS ITIMES IADD1 ISUB1 IDIFFERENCE IQUOTIENT IREMAINDER
         IEQN ILESSP IGREATERP IMINUSP COPY DELQIP LASTPAIR
         FLAG1 REMFLAG1 PLIST GETFN GETFNTYPE FNP EXPRP MACROP FEXPRP
         ERRSETT FEXPRCALL % DO
         INTERNP ULCASE CONCATIDL MAKEID MAKESTR MAKEINT MAKEFLO
         ORDERP LEXP STDINP STDOUTP FILEP PRIN2D PRIN1D
         UNREADCH SKIPTO BELL IOCHANS FILEIN FILEOUT
%        SETCHARFOLD CHARFOLD
         SETGCMESSAGE GCMESSAGE I!$PRNEVL QUIT BYE SHPDEBUG TIME)
       'LOSE) )  % These have been defined in the present file.

(eval!-when (eval compile)
 (IFP (getd 'tab) (FLAG '(TTAB) 'LOSE)) )

(eval!-when (eval compile)
 (IFP (getd 'rassq) (FLAG '(RATSOC) 'LOSE)) )

(INCLUDE "../exfn2i.sl")

(eval!-when (eval compile)
 (IFP (getd 'tab) (REMFLAG '(TTAB) 'LOSE)) )

(eval!-when (eval compile)
 (IFP (getd 'rassq) (REMFLAG '(RATSOC) 'LOSE)) )

(eval!-when (eval compile)
 (REMFLAG '(WARNING SKIPWHITE PGLINE EXPLODEC RESET)
          'LOSE)
 (REMFLAG '(EXPLODECHELP) 'LOSE)
 (REMFLAG '(EVENP XPLUS XTIMES XADD1 XSUB1 XDIFFERENCE XQUOTIENT
            XREMAINDER XEQN XLESSP XGREATERP XMINUSP AMEMQ AATSOC AEQN
            XIPLUS XITIMES XIADD1 XISUB1 XIDIFFERENCE XIQUOTIENT
            XIREMAINDER XIEQN XILESSP XIGREATERP XIMINUSP
            IPLUS ITIMES IADD1 ISUB1 IDIFFERENCE IQUOTIENT IREMAINDER
            IEQN ILESSP IGREATERP IMINUSP COPY DELQIP LASTPAIR
            FLAG1 REMFLAG1 PLIST GETFN GETFNTYPE FNP EXPRP MACROP FEXPRP
            ERRSETT FEXPRCALL % DO
            INTERNP ULCASE CONCATIDL MAKEID MAKESTR MAKEINT MAKEFLO
            ORDERP LEXP STDINP STDOUTP FILEP PRIN2D PRIN1D
            UNREADCH SKIPTO BELL IOCHANS FILEIN FILEOUT
%           SETCHARFOLD CHARFOLD
            SETGCMESSAGE GCMESSAGE I!$PRNEVL QUIT BYE SHPDEBUG TIME)
          'LOSE) )

% UNIX use lower case for system commands, file names etc.
(SETQ !*OSCASEUP NIL)

% Franz Lisp uses lower case internally.
(SETQ !*ILISPCASEUP NIL)

%(INCLUDE "systyp.sl")  % Moved to exfn2.sl. JEA.

(SETGLOBALS ARGV!*)

%(DE SLISPTOP () (ISLISPTOP))

(DE SET!-TOP!-LEVEL (X) (SETQ user!-top!-level X))

(DE INITCOMMANDLINE ()
 (IFORL X ((ISUB1 (argv -1)) 1 -1) (XPUSH ARGV!* (argv X)))
 ARGV!*)

(DM NEXTONCOMLIN (DUMMY)
 '(SLET ((AAAA ARGV!*))
   (COND (AAAA (SETQ ARGV!* (CDR ARGV!*))))
   AAAA) )

(DE SEXPRFROMCOM ()
 (SLET ((A (CHKERARGV)) B C)
  (WHILE
   (AND (NOT (PAIRP (SETQ B (ERRSET (REREADSTRING A) NIL))))
    (SETQ B (NEXTONCOMLIN)) )
   (SETQ A (CONCATID A " " (CAR B))) )
  (COND
   (B (CAR B))
   (T (REREADSTRING A)) ) ) )

(declare (special HERE HERELIST))

(eval!-when (eval compile)
 (DS INITSEARCHLIS ()
  (SLET ((A (argv 0)))          % This is for Sheep Lisp on Franz.
   (SETQ TOPDIR!* (COND
     ((NUMBERP A) (ERROR 0 "The name of the image must be available"))
     ((PREFIXOF A)) % Save for the benefit of *.ini files.
     (T '!. ) )) )) ) % Start from current directory.

(eval!-when (eval compile)
 (DS SETBATCH (QBT) (AND (NOT QBT) (status isatty)))
 (DS NEWLISPTOP () NIL)
 (SETQ !!COMMANDLINE T)
 (FLAG '(IFP) 'DFEXPAND)
 (FLAG '(NEWLISPTOP SETBATCH
         INITCOMMANDLINE NEXTONCOMLIN SEXPRFROMCOM
         SET!-TOP!-LEVEL INITSEARCHLIS)
       'LOSE) )  % These have been defined in the present file.

(INCLUDE "../slini.sl")

(eval!-when (eval compile)
 (REMFLAG '(NEWLISPTOP SETBATCH
            INITCOMMANDLINE NEXTONCOMLIN SEXPRFROMCOM
            SET!-TOP!-LEVEL INITSEARCHLIS)
          'LOSE) )

(DE DATETIME () (status ctime))

(SETGLOBALS
 (SAVEDO!*
  (LOADARG!* HERE HERELIST !*INT CLOC!*         % FIL!*
   gcdisable piport poport ARGV!* ) ) )

(declare (special NOSAVE !*IPRN))

(DFEXPR SAVERUN (X!&)
 (SETQ !*ERRMSG (SETQ !*IPRN T))
 (FOREACH Y IN SAVEDO!* DO (COND ((PAIRP Y) (EVAL Y)) (T (STNIL Y))))
 (SETQ MODULNAM!* 'UNKNOWN)
 (SETQ LOADMSGTAB!* 0) 
 (SETQ PGnum!* 1)
 (SETQ LInum!* 1)
 (SETQ user!-top!-level 'SLISPINIT)
 (resetio)
 (gc)
 (COND
  ((AND (BOUNDP 'NOSAVE) NOSAVE)
   (PRIN2T "Ready to be saved") )
  (T (PRIN2T "About to save")
     (funcall 'dumplisp (NCONS (SETCASE X!&))) )) )

(DC D INITLINEFN () (DEBUGGING T T) NIL)

% Earlier called -d. JEA
(DC A INITLINEFN () (DEBUGGING T NIL) NIL)

% Earlier called -h. JEA
(DC S INITLINEFN () (SETQ !*OUTSLSH T) NIL)

% Earlier called -H. JEA
(DC R INITLINEFN () (SETQ !*OUTSLSH NIL))

(COMMENT                        % This is only used to debug SLISPINIT
(DE DEBUGTOP () (SETQ user!-top!-level NIL) (reset))
(DEFLIST '((k DEBUGTOP) (K DEBUGTOP)) 'INITLINEFN) )

%### EXFN2.SL end
