;### SLFUNS.L =======================================================
;
;       SLFUNS.L
;       --------
;
;       Copyright (c) 1982 by Inge Frick, Institute of Theoretical
;       Physics, Vanadisvagen 9, S-113 46 Stockholm, Sweden.
;
;       This file defines the Sheep Lisp functions in upper case
;       with the help of Franz lisp functions in lower case. Some
;       functions are different for compiler and interpreter. These
;       functions are defined in SLISP.L and SLMAC.L .
;       Some features in this file appear in two versions depending on
;       which version of Franz Lisp they should be used. Look for the
;       macro IFP.
;
;====================================================================

(declare (special $EOF$ $EOL$ EMSG* *ERRMSG *BAKGAG *ECHO *MSG *RAISE
                  T LInum* PGnum* FRANZREADTABLE SLISPREADTABLE
                  XSLISPREADTABLE *OUTSLSH)
         (special piport poport readtable Standard-Input Standard-Output
                  $gcprint trace-printer showstack-printer linel
                  ER%brk ER%misc ER%err ER%undef ER%all ER%tpl)
         (localf remescc Noider tYPCVT Errprin Xndf xTYI PrinX PrinXe
                 patomX patVec GTpnm nilutl GTpnmake DOFITINLINE)
         (macros t))

(defun Sameas (newname oldname)
  (Sameas1 newname oldname)
  (Sameas1 oldname newname) )

(defun Sameas1 (newname oldname)
  (putprop newname oldname 'GTrnam)
  (putprop oldname newname 'GTpnm) )

;IFP is defined here to allow some conditional coding
(defun IFP macro (u) (cond
 ((eval (cadr u)) (caddr u))
 ((cdddr u) (cadddr u)) ))

;MAHM's fixes by copying from Franz sources
;--- rassq : like assq but look at the cdr instead of the car
(IFP (not (getd 'rassq))
 (def rassq
   (lambda (form list)
      (cond ((null list) nil)
            ((not (dtpr list))
             (error "rassq: illegal second argument: " list))
            (t (do ((ll list (cdr ll)))
                   ((null ll) nil)
                   (cond ((eq form (cdar ll)) (return (car ll))))))))) )

; function >& undefined in opus 36, use >. JEA
(IFP (not (getd 'tab))
 (defun tab n
   (prog (nn prt over)
      (setq nn (arg 1))
      (cond ((IFP (getd '=&) (>& n 1) (> n 1))
             (setq prt (arg 2))))
      (cond ((IFP (getd '=&)
                  (>& (setq over (nwritn prt)) nn)
                  (> (setq over (nwritn prt)) nn) )
             (terpri prt)
             (setq over 0)))
      (printblanks (- nn over) prt))) )
;End of the MAHM fixes

; 3.1 Elementary Predicates

(mapcar 'samedef
 '(CODEP EQ EQUAL FIXP FLOATP IDP     NULL NUMBERP PAIRP STRINGP)
 '(bcdp  eq equal fixp floatp symbolp null numberp dtpr  stringp))

(defun ATOM (x) (not (dtpr x)))

(defun CONSTANTP (x) (not (or (dtpr x) (symbolp x))))

(samedefX 'EQN 'equal)


; 3.2 Functions on Dotted-Pairs

(mapcar 'Sameas                 ; This is necessary for the compiler.
  '(CAR CDR CAAR CADR CDAR CDDR)
  '(car cdr caar cadr cdar cddr))

(mapcar 'samedefX
  '(CAAAR CAADR CADAR CADDR CDAAR CDADR CDDAR CDDDR
  CAAAAR CAAADR CAADAR CAADDR CADAAR CADADR CADDAR CADDDR
  CDAAAR CDAADR CDADAR CDADDR CDDAAR CDDADR CDDDAR CDDDDR)
  '(caaar caadr cadar caddr cdaar cdadr cddar cdddr
  caaaar caaadr caadar caaddr cadaar cadadr caddar cadddr
  cdaaar cdaadr cdadar cdaddr cddaar cddadr cdddar cddddr))

(mapcar 'samedef
  '(CONS LIST RPLACA RPLACD)
  '(cons list rplaca rplacd))


; 3.3 Identifiers

(declare (special Dummyiport Dummyoport))

(defun GENSYM () (gensym 'G))

(defun COMPRESS (x)
 (cond
  ((or (eq (car x) '\!) (LITER (car x)))
   (maknam (remescc x)) )
  (t (let ((piport Dummyiport) (poport Dummyoport)) (readlist x))) ))

(defun remescc (x) (cond
  ((not (and x (cdr x))) x)
  (t (cond ((eq (car x) '\!) (setq x (cdr x))))
     (cons (car x) (remescc (cdr x))) ) ))

(defun EXPLODE (x)
 (IFP (getd 'getsyntax)
  (aexplode (cond
    ((symbolp x) (GTpnm x))
    (t x) ))
  (cond
   ((and (symbolp x) (dtpr (setq x (GTpnm x)))) (mapcar 'ascii x))
   (t (aexplode x)) ) ) )

(defun INTERN (x)
 (let ((y (concat x)))
  (cond
   ((eq y x) x)
   (t (NILutls y)) ) ) )

(samedef 'REMOB 'remob)


; 3.4 Property List Functions

(defun GET (x y) (cond
  ((symbolp x) (get x y)) ))

(defun PUT (x y z) (cond
  ((not (symbolp x)) (Noider x))
  ((not (symbolp y)) (Noider y))
  (t (putprop x z y)) ))

(defun REMPROP (x y) (cond
  ((and (symbolp x) (setq x (remprop x y))) (car x))
  (t nil) ))

(samedefX 'FLAGP 'GET)

(defun FLAG (X y) (do x X (cdr x) (null x) (PUT (car x) y 'IsFlag)))

(defun REMFLAG (X y) (do x X (cdr x) (null x) (REMPROP (car x) y)))

(defun Noider (x) (ERROR 0 (list x "is not an identifier")))


; 3.5 Function Definition

(defun DE macro (x) (Xndf x 'EXPR))

(defun DF macro (x) (Xndf x 'FEXPR))

(defun DM macro (x) (Xndf x 'MACRO))

(defun Xndf (x y) `(PUTD ',(cadr x) ',y ',(cons 'lambda (cddr x))))

(defun GETD (x)
 (prog (y z u)
  (cond
   ((not (and (symbolp x) (setq y (getd x)))) (return nil))
   ((dtpr y)
    (cond ((eq (setq z (car y)) 'lambda) (return (cons 'EXPR y)))
          ((setq u (get x 'FNtyP)) (return u))
          (t (setq u (cons 'lambda (cdr y)))) ) )
   ((not (bcdp y)) (return nil))
   ((eq (setq z (getdisc y)) 'lambda) (return (cons 'EXPR y)))
   ((setq u (get x 'FNtyP)) (return u))
   (t (setq u (mfunction nil nil))      ; Has to be done
      (rplacd u (cdr y))                ; this way because of
      (rplaca u 'lambda) ) )            ; of bug in mfunction
  (setq z (cond
    ((eq z 'nlambda) 'FEXPR)
    ((eq z 'macro) 'MACRO)
    (t 'EXPR) ))                        ; Must be a foreign function
  (return (putprop x (cons z u) 'FNtyP)) ))

(defun REMD (x) (prog1 (GETD x) (remprop x 'FNtyP) (putd x nil)))

(defun PUTD (x y z) (cond
  ((GET x 'LOSE) (WARNING (list x "LOSEd")) nil)
  (t (cond ((and *MSG (getd x)) (WARNING (list x "being redefined"))))
     (setq y (cond ((GET y 'fnTYP))
                   (t (ERROR 0 (list y "is not a function type"))) ))
     (remprop x 'FNtyP)
     (putd x (cond
       ((dtpr z) (cond ((eq y (car z)) z) (t (cons y (cdr z)))))
       ((eq y (getdisc z)) z)
       (t (let ((u (mfunction nil nil))); Fix for bug in mfunction
               (rplacd u (cdr z))       ;
               (rplaca u y) )) ))       ;
     x ) ))

(putprop 'EXPR 'lambda 'fnTYP)

(putprop 'FEXPR 'nlambda 'fnTYP)

(putprop 'MACRO 'macro 'fnTYP)


; 3.6 Variables and Bindings

(mapcar 'samedef
 '(SET SETQ)
 '(set setq))

(defun FLUID (u)
 (cond ((getd 'liszt) (funcall 'special u)))
 (mapc
  '(lambda (x)
    (cond
     ((GLOBALP x)
      (cond ((not (get x 'SLglob-flu)) (putprop x 'FLUID 'SLglob-flu)))
      (WARNING (list x "is GLOBAL cannot be FLUID")) )
     (t (PUT x 'SLglob-flu 'FLUID)) )
    (QSET x nil) )
  u) nil)

(defun UNFLUID (u)
 (mapc
  '(lambda (x) (cond
     ((and (GET x 'SLglob-flu) (not (GLOBALP x)))
      (cond ((getd 'liszt) (funcall 'unspecial (ncons x))))
      (remprop x 'SLglob-flu))
     (t (WARNING (list x "is not a FLUID cannot be UNFLUID"))) ))
  u) nil)

(defun GLOBAL (u)
 (cond ((getd 'liszt) (funcall 'special u)))
 (mapc
  '(lambda (x)
    (PUT x 'SLglob-flu 'GLOBAL)
    (QSET x nil) )
  u) nil)

(defun QSET (x y) (cond ((not (boundp x)) (set x y))))

(defun GLOBALP (x)
 (and (symbolp x) (or (getd x) (eq (get x 'SLglob-flu) 'GLOBAL))) )

(defun FLUIDP (x) (eq (GET x 'SLglob-flu) 'FLUID))


; 3.7 Program Feature Functions

(mapcar 'samedef
 '(GO PROG PROGN RETURN PROG2)
 '(go prog progn return prog2))


; 3.8 Error Handling

(IFP (getd 'err-with-message)       ; For Franz with opus>=38.77. JEA
 (defun ERROR (x y) (progn          ; JEA
   (setq EMSG* y)                   ; JEA
   (err-with-message nil x) ))      ; JEA
 (defun ERROR (x y) (progn
   (setq EMSG* y)
   (err x) )) )

(defun ERRORSET (x *ERRMSG *BAKGAG)
 (errset (eval x) nil) )

(defun Errprin (x y poport)
  (terpri)
  (patom x)
  (cond
   ((not (dtpr y)) (PRIN2 y))
   (t (do ((z y (cdr z))) ((not (dtpr z)) (cond (z (PRIN1 z))))
          (PRIN2 (car z)) (tyo 32) ) ) )
  (terpri) )


; 3.9 Vectors

(defun VECTORP (x)
 (or (hunkp x) (and (arrayp x) (eq (getaux x) 'Slisp))) )

(defun MKVECT (x) (cond
  ((|<| (setq x (|1+| x)) 128) (makhunk x))
  (t (do ((u (marray
              (small-segment 'value x)
              (getd 'arrac-oneD) 'Slisp x 4) )
          (v (|1-| x) (|1-| v)) )
         ((minusp v) u)
         (set (arrayref u v) nil) )) ))

(eval-when (compile)
 (defmacro value-eval (x) `(cdr ,x)))

(eval-when (eval)
 (defun value-eval (x) (eval x)))

(defun GETV (x y) (cond
  ((hunkp x) (cxr y x))
  (t (value-eval (arrayref x y))) ))

(defun PUTV (x y z) (cond
  ((hunkp x) (rplacx y x z) z)
  (t (set (arrayref x y) z)) ))

(defun UPBV (x) (cond
  ((hunkp x) (|1-| (hunksize x)))
  ((arrayp x) (|1-| (getlength x))) ))

(defun rVect nil
 (do ((x (ncons (READ)) (cons (READ) x)) (z (xTYI) (xTYI)))
     ((eq z 93)
      (setq x (nreverse x))
      (setq z (length x))
      (cond
       ((|<| z 128) (apply 'hunk x))
       (t (setq z (marray
                   (small-segment 'value z)
                   (getd 'arrac-oneD)
                   'Slisp z 4 ))
          (fillarray z x) ) ) )
     (cond ((not (eq z 44)) (ERROR 0 "Badly formed vector"))) ) )

(defun xTYI ()          ; skip past white space
 (do ((x (TYI) (TYI))) ((not (memq x '(32 9 13 10 12))) x)) )

(defun TYI ()
 (let ((x (tyi)))
  (cond (*ECHO (tyo x)))
  (cond
   ((eq x 7) (rBELL))
   ((eq x 10) (rLF))
   ((eq x 12) (rFF)) )
  x) )

; 3.10 Boolean Functions and Conditionals

(mapcar 'samedef
 '(AND OR NOT COND)
 '(and or not cond))


; 3.11 Arithmetic Functions

(mapcar 'samedef
 '(ABS DIFFERENCE EXPT FIX FLOAT GREATERP LESSP MAX MIN
   MINUS PLUS QUOTIENT REMAINDER TIMES
   ADD1 SUB1 ONEP MINUSP)
 '(abs *dif       expt fix float greaterp lessp max min
   minus plus quotient remainder times
   add1 sub1 onep minusp))

(samedef 'ZEROP (IFP (getd 'any-zerop) 'any-zerop 'zerop))

(mapcar 'samedefX
 '(MAX2 MIN2 PLUS2 TIMES2)
 '(max  min  plus  times ) )

(defun DIVIDE (x y)
 (setq x (Divide x y))
 (rplacd x (cadr x)) )


; 3.12 MAP Composite Functions

(defun MAP (MLIST FN) (map FN MLIST) nil)

(defun MAPC (MLIST FN) (mapc FN MLIST) nil)

(defun MAPCAN (MLIST FN) (mapcan FN MLIST))

(defun MAPCAR (MLIST FN) (mapcar FN MLIST))

(defun MAPCON (MLIST FN) (mapcon FN MLIST))

(defun MAPLIST (MLIST FN) (maplist FN MLIST))


; 3.13 Composite Functions

(mapcar 'samedef
 '(APPEND NCONC ASSOC LENGTH MEMBER MEMQ REVERSE SASSOC SUBST)
 '(append nconc assoc length member memq reverse sassoc subst))

(defun DEFLIST (x y) (mapcar
  '(lambda (z) (PUT (car z) y (cadr z)) (car z))
  x ))

(defun DELETE (x y) (cond
  ((null y) nil)
  ((equal (car y) x) (cdr y))
  (t (cons (car y) (DELETE x (cdr y)))) ))

(defun PAIR (x y) (cond
  ((and x y) (cons (cons (car x) (car y)) (PAIR (cdr x) (cdr y))))
  ((or x y) (ERROR 0 "Different length lists in PAIR")) ))

(defun SUBLIS (x y)
 (cond
  (x (let ((u (assoc y x)))
      (cond
       (u (cdr u))
       ((dtpr y) (cons (SUBLIS x (car y)) (SUBLIS x (cdr y))))
       (t y) ) ))
  (t y) ) )

(defun DIGIT (x) (GET x 'Digit))

(defun LITER (x) (GET x 'Letter))

(mapc '(lambda (x) (putprop x x 'Digit))
  '(\0 \1 \2 \3 \4 \5 \6 \7 \8 \9) )

(mapc '(lambda (x y) (putprop x x 'Letter) (putprop y x 'Letter))
  '(A B C D E F G H I J K L M N O P Q R S T U V W X Y Z)
  '(a b c d e f g h i j k l m n o p q r s t u v w x y z) )


; 3.14 The Interpreter

(samedef 'EVAL 'eval)

(samedef 'APPLY 'apply)

(defun EXPAND (x fn) (cond
 ((null (cdr x)) (car x))
 (t (list fn (car x) (EXPAND (cdr x) fn))) ))

; Try (samedefX 'FUNCTION 'quote) for Franz opus 36 or lower.
; Use (status version) as test ?   Seems to be ok also for opus 36. JEA
(samedefX 'FUNCTION 'function)

(Sameas1 'QUOTE 'quote)

(defun EVLIS (x) (mapcar 'eval x))


; 3.15 Input and Output

(defun OPEN (x y) (cond
  ((eq y 'INPUT) (infile x))
  ((eq y 'OUTPUT) (outfile x))
  (t (ERROR 0 (list y "is not option for OPEN"))) ))

(samedef 'CLOSE 'close)

(defun RDS (x) (prog1 piport (setq piport x)))

(defun WRS (x) (prog1 poport (setq poport x)))

(declare (special **pr*))

(defun PRIN1 (x) (let ((**pr* nil)) (PrinX x) x))

(defun PRINT (x) (let ((**pr* nil)) (PrinX x) (terpri) x))

(defun PRIN2 (x) (let ((**pr* t)) (PrinX x) x))

(defun PRINC (x)
 (cond
  ((eq x $EOL$) (terpri))
  (t (princ x)) )
 x )

(defun PrinX (x) (cond
  ((not (dtpr x)) (patomX x))
  (t (tyo 40) (PrinX (car x)) (PrinXe (cdr x)) (tyo 41)) ))

(defun PrinXe (x) (cond
  (x (tyo 32) (cond
    ((not (dtpr x)) (tyo 46) (tyo 32) (patomX x))
    (t (PrinX (car x)) (PrinXe (cdr x))) )) ))

(defun patomX (x)
 (let ((readtable (cond (*OUTSLSH XSLISPREADTABLE) (t SLISPREADTABLE))))
  (cond
   ((VECTORP x) (patVec x))
   ((not (symbolp x))
    (DOFITINLINE (flatc x))
    (cond (**pr* (patom x)) (t (print x))) )
   (**pr* (DOFITINLINE (flatc (setq x (OUTpnm x)))) (patom x))
   (t (IFP (getd 'getsyntax)
       (progn (setq x (GTpnm x)) (DOFITINLINE (pntlen x)) (print x))
       (cond
        ((dtpr (setq x (GTpnm x)))
         (DOFITINLINE (length x))
         (mapc 'tyo x) )
        (t (DOFITINLINE (pntlen x)) (print x)) ) )) ) ) )

(declare (unspecial **pr*))

(defun DOFITINLINE (y) (let ((z (nwritn)))
  (cond ((not (or (zerop z) (|<| (|+| y z) linel))) (terpri))) ))

(defun patVec (x)
 (tyo 91)
 (PrinX (GETV x 0))
 (let ((u (|1+| (UPBV x))))
  (do a 1 (|1+| a) (|=| a u) (tyo 44) (PrinX (GETV x a))) )
 (tyo 93) )

(defun GTpnm (x) (cond
  ((get x 'GTpnm))
  (t x) ))

(defun OUTpnm (x)
 (IFP (getd 'getsyntax)
  (GTpnm x)
  (let ((y (GTpnm x))) (cond ((listp y) x) (t y))) ) )

(defun nilutl (x) (cond
 ((dtpr x) (rplaca x (nilutl (car x))) (rplacd x (nilutl (cdr x))))
 ((VECTORP x) (do ((n (UPBV x) (|1-| n))) ((minusp n) x)
                  (PUTV x n (nilutl (GETV x n))) ))
 ((not (symbolp x)) x)
 (t (NILutls (cond (*RAISE (Ukey x)) (t x)))) ))

(IFP (not (getd 'concatl)) (defun concatl (x) (apply 'concat x)))

(defun Ukey (x) (concatl (UKEYL (aexplodec x))))

(defun UKEYL (l) (map
  (function (lambda (x)
    (let ((a (get (car x) 'Letter))) (cond (a (rplaca x a)))) ))
  l))

(defun NILutls (x) (cond
 ((eq x 'NIL) nil)
 ((get x 'GTrnam))
 (t (IFP (getd 'getsyntax) x (progn (GTpnmake x) x))) ))

(IFP (not (getd 'getsyntax)) (progn 'compile
  (defun GTpnmake (x) (do
    ((z) (w (aexploden x) (cdr w)) (u) (y))
    ((null w)
     (cond (z (putprop x
        (cond
         ((or (eq (car (setq y (nreverse y))) 33) (|>| (car y) 64)) y)
         (t (cons 33 y)) )
        'GTpnm))) )
    (setq u (car w))
    (setq y (cons u (cond
       ((or (eq u 43) (eq u 45)) (setq z t) (cons 33 y))
       ((or
         (|<| u 48)
         (|>| u 122)
         (and (|<| 57 u) (|<| u 65))
         (and (|<| 90 u) (|<| u 97)) )
        (cons 33 y) )
       (t y) ))) ))

  (INTERN "+") (INTERN "-")     ;Initialise GTpnm for + and -
))

(defun READ ()
; (PROMPT PROMPTSTRING*)
 (let ((x (nilutl (read piport $EOF$))))
  (cond ((and *ECHO (not (eq x $EOF$))) (PRINT x)))
  x ) )

(defun READCHnl ()
; (cond ((eq (nwritn Standard-Input) 0) (PROMPT PROMPTSTRING*)))
 (let ((x (readc piport $EOF$)))
  (cond ((and *ECHO (not (eq x $EOF$))) (princ x)))
  (cond
   ((eq x '#.(ascii 7)) (rBELL))
   ((eq x '#.(ascii 10)) (rLF))
   ((eq x '#.(ascii 12)) (rFF)) )
  x ) )

(defun READCH ()
 (let ((x (READCHnl)))
  (cond ((and *RAISE (get x 'Letter)))
        (t x) ) ) )

(samedef 'TERPRI 'terpri)

(defun EJECT () (progn (tyo 12.) (drain)))

(defun LINELENGTH (x) (prog1 linel
  (cond (x (cond
    ((and (fixp x) (plusp x) (|<| x 512.)) (setq linel x))
    (t (ERROR 0 (list x "is an invalid LINELENGTH"))) ))) ))

(samedef 'POSN 'nwritn)

(defun LPOSN () 0)

(defun PAGELENGTH (n) "There is no line-counting in this version")


; 4. System Global Variables

(Sameas 'LAMBDA 'lambda)

(GLOBAL '($EOF$ $EOL$ EMSG* T nil))

(FLUID '(*GC *ERRMSG *RAISE *COMP *ECHO *BAKGAG *MSG *OUTSLSH))

(Sameas '*GC '$gcprint)

(Sameas1 'NIL 'nil)

(setq $EOL$ '|
|)

(setq $EOF$   (ascii 127)
      T       'T
      *BAKGAG 0
      *ERRMSG t)

;Error handling

(def Slerror (lexpr (n) (ERROR 0 (listify n))))

(putd 'Flerror (getd 'error))

(putd 'error (getd 'Slerror))

(defun Baktrace ()
 (let ((piport nil) (poport nil))
  (cond ((status isatty)
         (drain Standard-Input) ))      ; remove type-ahead on tty
  (cond (EMSG* (Errprin "***** " EMSG* nil)))
  (cond (*BAKGAG
    (cond
     ((not (numberp *BAKGAG)) (funcall 'baktrace nil))
     ((zerop *BAKGAG) (funcall 'showstack nil))
     ((minusp *BAKGAG) (funcall 'showstack (list (abs *BAKGAG))))
     (t (funcall 'baktrace (list *BAKGAG))) ) ))
  nil ) )

(defun ER%misc-Handler (x)
 (progn (setq EMSG* (cdddr x))
  (cond (*ERRMSG (Baktrace))) ) )

(defun ER%undef-Handler (x) (cond
  ((let ((poport nil)) (undef-func-handler x)))
  (t (ER%misc-Handler x)) ))

(defun ER%err-Handler (x) (cond
 ((and *ERRMSG (not (eq (cadddr x) $EOF$))) (Baktrace)) ))

(defun RESET () (setq piport (setq poport nil)) (resetio) (reset))

(defun RESET1 (dummy) (RESET))

(defun DEBUGGING (x y)
 (apply 'sstatus (list 'translink (not x)))
 (signal 2 (cond (x 'sys:int-serv) (t 'RESET1)))
 (cond
  (y (setq ER%brk       nil
           ER%misc      nil
           ER%err       nil
           ER%undef     'undef-func-handler
           trace-printer 'Trace-print
           showstack-printer 'print))
  (t (setq ER%brk       'ER%misc-Handler
           ER%misc      'ER%misc-Handler
           ER%err       'ER%err-Handler
           ER%undef     'ER%undef-Handler
           trace-printer 'PRIN1
           showstack-printer 'PRIN1)) )
 (cond
  (x  (setq ER%all      'debug-err-handler
            ER%tpl      'break-err-handler ))
  (t  (setq ER%all      nil
            ER%tpl      'RESET1 )) )
 (*rset x) )

(setq $gcprint t)

(sstatus ignoreeof t)

(sstatus automatic-reset t)

(DEBUGGING nil nil)

(LINELENGTH 76)

; Extras.

(setq PGnum*            1
      LInum*            1)

(defun rFF ()
 (setq LInum* 0)
 (setq PGnum* (|1+| PGnum*))
 nil)

(defun rLF ()
 (setq LInum* (|1+| LInum*))
 nil)

(defun rBELL () (RESET))

(defun PGLINE () (cons PGnum* LInum*))

(defun WARNING (x) (cond (*MSG (Errprin "*** " x nil))))

(defun SKIPWHITE () ; skip past white space
 (IFP (getd 'untyi)
  (untyi (xTYI))
  (cond ((memq (tyipeek) '(32 9 13 10 12)) (TYI) (SKIPWHITE))) ) )

(defun EXPLODEC (x)
 (aexplodec (cond
    ((symbolp x) (OUTpnm x))
    (t x) )))

(defun REREADSTRING (x) (nilutl (readlist (explodec x))))

(defun LKEY (x)
 (concatl (map
   (function (lambda (x)
     (let ((a (car x)))
      (cond ((and (|<| 64 a) (|<| a 91)) (setq a (|+| a 32))))
      (rplaca x (ascii a)) ) ))
   (aexploden x) )) )

(cond ((not (getd 'Iload)) (putd 'Iload (getd 'load))))

(defun Fload (x) (let ((readtable FRANZREADTABLE)) (Iload x)))

;### SLFUNS.L end
