%### EXFN2.SL =======================================================
%
%       EXFN2.SL
%       --------
%
%       Copyright (c) 1988, 1998, 2008 by Jan E. Aman and Inge Frick,
%       Fysikum, Stockholm University, S-106 91  Stockholm, Sweden.
%
%       This file contains some changes to EXFN2I.SL that adapts it to
%       PSL.
%
%====================================================================

(SETGLOBALS
 ((TAB!*        (INT2ID 9)))    % Tab.
 IMGFILE!*
 TOPDIR!*
 INITPROMPT!*
 TOPLEVEL!*
 !!COMMANDLINE
 (SAVEDO!*
  (LOADARG!* HERE HERELIST !*INT CLOC!*         % FIL!*
   COMMANDLINE!* ARGV!* ARGV0!* CURRENTDIR!* ) ) )

(SETFLUIDS
 YESPDEFAULT!* NOPDEFAULT!* NOSAVE !*IPRN
 (LOADMSGTAB!* 1)               % Position for Load message.
 (MODULNAM!* UNKNOWN) )         % Module name in LOADX.

% The following functions with prefix I and A operate on integers with
% absolute value < 1000. On many systems this means EQ can be used.
% Those with prefix A will also handle any argument but only guarantee
% to test equality when one argument is a small integer. Those with
% prefix I might give anything when an argument is not a small integer.
% Use I when both arguments are known to be small.
% Use A when one argument is known to be small.

(SAMEDEF AMEMQ MEMQ)

(SAMEDEF AATSOC ATSOC)

(SAMEDEF AEQN EQ)

% Compilation of BUWEYLMF in module invrf requires at least one (?)
% of the CMACRO definitions of ITIMES, IPLUS or IMINUS to be omitted
% for VMS PSL.
(IFP (NOT (EQ 'VMS (SYSTYPE 'OS)))
 (PUT 'IPLUS 'CMACRO '(LAMBDA (A B) (WPLUS2 A B))) )
(PUT 'ITIMES 'CMACRO '(LAMBDA (A B) (WTIMES2 A B)))
(PUT 'IADD1 'CMACRO '(LAMBDA (A) (WPLUS2 A 1)))
(PUT 'ISUB1 'CMACRO '(LAMBDA (A) (WDIFFERENCE A 1)))
(PUT 'IMINUS 'CMACRO '(LAMBDA (A) (WDIFFERENCE 0 A)))
(PUT 'IDIFFERENCE 'CMACRO '(LAMBDA (A B) (WDIFFERENCE A B)))
(PUT 'IQUOTIENT 'CMACRO '(LAMBDA (A B) (WQUOTIENT A B)))
(PUT 'IREMAINDER 'CMACRO '(LAMBDA (A B) (WREMAINDER A B)))

(SAMEDEF IEQN EQ)

(PUT 'ILESSP 'CMACRO '(LAMBDA (A B) (WLESSP A B)))
(PUT 'IGREATERP 'CMACRO '(LAMBDA (A B) (WGREATERP A B)))

% The following functions with prefix XI and XA operate on integers
% with absolute value < 64000. On some systems this means EQ can be
% used. Those with prefix XA will also handle any argument but only
% guarantee to test equality when one argument is a small integer.
% Those with prefix I might give anything when an argument is not
% a small integer. Use XI when both arguments are known to be small.
% Use XA when one argument is known to be small.

(SAMEDEF XAMEMQ MEMQ)

(SAMEDEF XAATSOC ATSOC)

(SAMEDEF XAEQN EQ)

% To avoid "Compiler bug: expression to complicated" when compiling
% sheep functions SFIN2, AFIN2, HFIN2 in module karlim the CMACRO
% definition of either XIPLUS or XIDIFFERENCE must be omitted for
% VMS PSL.
(IFP (NOT (EQ 'VMS (SYSTYPE 'OS)))
 (PUT 'XIPLUS 'CMACRO '(LAMBDA (A B) (WPLUS2 A B))) )
(PUT 'XITIMES 'CMACRO '(LAMBDA (A B) (WTIMES2 A B)))
(PUT 'XIADD1 'CMACRO '(LAMBDA (A) (WPLUS2 A 1)))
(PUT 'XISUB1 'CMACRO '(LAMBDA (A) (WDIFFERENCE A 1)))
(PUT 'XIMINUS 'CMACRO '(LAMBDA (A) (WDIFFERENCE 0 A)))
(PUT 'XIDIFFERENCE 'CMACRO '(LAMBDA (A B) (WDIFFERENCE A B)))
(PUT 'XIQUOTIENT 'CMACRO '(LAMBDA (A B) (WQUOTIENT A B)))
(PUT 'XIREMAINDER 'CMACRO '(LAMBDA (A B) (WREMAINDER A B)))

(SAMEDEF XIEQN EQ)

(PUT 'XILESSP 'CMACRO '(LAMBDA (A B) (WLESSP A B)))
(PUT 'XIGREATERP 'CMACRO '(LAMBDA (A B) (WGREATERP A B)))

% Warning! PSL have two incompatible definitions of PLIST. In COMMON it
% is defined as a macro calling PROP (as below), in DEBUG it is instead
% defined to CALL EVPLIST to print list. This version should not have
% argument quoted as opposite to the definition in COMMON.
(SAMEDEF PLIST PROP)
(COMPILETIME (FLAG '(PLIST) 'LOSE))     % To avoid the DEBUG definition.

(DS FOREACHID (X !&REST L) (MAPOBL (FUNCTION (LAMBDA (X) . L))))

% ERRSETT([ARG:form]):any
% ERRSETT evaluates expressions ARG in an ERRSET. Error messages are
% turned on if not a surrounding ERRSET has turned them off.
(DMACRO ERRSETT (!&REST U) (LIST 'ERRSET (MKPROGN U) '!*EMSGP))

(IFP (MEMQ 'COMMON OPTIONS!*) NIL (SAMEDEF EXPLODEC EXPLODE2))

% CONCATIDL MAKEID MAKESTR MAKEINT MAKEFLO

(DE LEXP (X Y)
 (SLET
  ((X1 (LISP2CHAR X)) (Y1 (LISP2CHAR Y)))
  (COND
   ((LESSP X1 Y1) T)
   ((GREATERP X1 Y1) NIL)
   (T
    (ORDLP
     (CDR (STRING2LIST (ID2STRING X)))
     (CDR (STRING2LIST (ID2STRING Y))) ) ) ) ) )

% ORDLP compares two lists of numbers, returns NIL if L2 comes
% before L1 lexicographically, otherwise T. From module hlpfns.
(DE ORDLP (L1 L2) (COND
  ((NULL L1) T)
  ((NULL L2) NIL)
  ((XIGREATERP (CAR L2) (CAR L1)) T)
  ((XILESSP (CAR L2) (CAR L1)) NIL)
  (T (ORDLP (CDR L1) (CDR L2))) ))

(DE ORDERP (X Y) (NOT (IGREATERP (ID2INT X) (ID2INT Y))))

(DE STDINP () (EQ IN!* STDIN!*))

(DE STDOUTP () (EQ OUT!* STDOUT!*))

% Provisional fix for VMS only:
% As the PSL function FLUSHBUFFER is incorrectly defined in the VMS
% version (an end of line is always added) it is in most cases
% preferable for Sheep not to flush the buffer. This means that
% messages like "rie  made" will normally not appear at all until
% that line is completed, however if there will be an interrupt by
% other output like gc-message the part "rie " will at that time be
% printed (flushed) and " made" will appear later as normal for Sheep.
% To have messages immediately flushed type '(ON FLUSH)'.
(IFP (EQ 'VMS (SYSTYPE 'OS))
 (CPROGN
  (SETGLOBALS SWITCHES)
  (SETFLUIDS FLUSH)
  (ADDHLP 'SWITCHES 'FLUSH
    "Flush print buffer immediately (in spite of bug in VMS-PSL)" )
  (DE PRIN2D (X) (PRIN2 X) (COND (FLUSH (FLUSHBUFFER OUT!*))) X)
  (DE PRIN1D (X) (PRIN1 X) (COND (FLUSH (FLUSHBUFFER OUT!*))) X) )
 (CPROGN
  (DE PRIN2D (X) (PRIN2 X) (FLUSHBUFFER OUT!*) X)       % Non-VMS
  (DE PRIN1D (X) (PRIN1 X) (FLUSHBUFFER OUT!*) X) ) )   % Non-VMS

% Compilation of SKIPWHITE gives serious trouble on SUN-3.
(IFP
 (OR
  (EQ 'VMS (SYSTYPE 'OS))
% (AND                          % Testing.
%  (EQ 'SUN (SYSTYPE 'MACHINE))
%  (NOT (EQN 3 (SYSTYPE 'MACHINEVERS))) )
  )
 (DE UNREADCH (CH) (UNREADCHAR (LISP2CHAR CH))) )

% SKIPWHITE SKIPTO

(SAMEDEF TTAB TAB)

% Note! TYO is an undocumented function in PSL.
(DE BELL NIL (TYO 7))

(DE PGLINE NIL (CONS (POSN) (LPOSN)))

(IFP
 (AND
  (NUMBERP (SYSTYPE 'LISPVERS))
  (GREATERP (SYSTYPE 'LISPVERS) 3.3) )
 (CPROGN
% For newer PSL-versions:

% FILEOUT(FIL:file,[ARG:form]):any
% Evaluates ARGs with output switched to file FIL.
  (DS FILEOUT (X !&REST L)
   (SLET ((OUTCH!$!$ (CAR (XPUSH FILESTK!* (OPEN X 'OUTPUT)))))
    (CHANNELLINELENGTH OUTCH!$!$ FILELINELENGTH!*)
    (PROG1
     (CHANOUT OUTCH!$!$ . L)
     (CLOSE (POP FILESTK!*)) ) ) )

  )

 (CPROGN
% For older PSL-versions:
  (DE IOCHANS (CHI CHO BODY)
   (SLET (AL PL)
    (COND (CHI (XPUSH PL 'IN!*) (XPUSH AL (CAR CHI))))
    (COND (CHO (XPUSH PL 'OUT!*) (XPUSH AL (CAR CHO))))
    (CONS (CONSA 'LAMBDA PL BODY) AL) ) )

  (DS STDIN (!&REST L) ((LAMBDA (IN!*) . L) STDIN!*))

  (DS STDOUT (!&REST L) ((LAMBDA (OUT!*) . L) STDOUT!*))

  (DS STDINOUT (!&REST L) ((LAMBDA (IN!* OUT!*) . L) STDIN!* STDOUT!*))

  (DS FILEIN (X !&REST L)
   (SLET ((INCH!$!$ (OPEN X 'INPUT)) RES!$!$)
    (UNWIND!-PROTECT
     (SETQ RES!$!$ (CHANIN INCH!$!$ . L))
     (CLOSE INCH!$!$) )
    RES!$!$) )

  (DS FILEOUT (X !&REST L)
   (SLET ((OUTCH!$!$ (OPEN X 'OUTPUT)) RES!$!$)
    (CHANNELLINELENGTH OUTCH!$!$ FILELINELENGTH!*)
    (UNWIND!-PROTECT
     (SETQ RES!$!$ (CHANOUT OUTCH!$!$ . L))
     (CLOSE OUTCH!$!$) )
    RES!$!$) )

  ))

% PROMPT is a noop as PSL prompts.
(DE PROMPT () NIL)

% SETFOLDCASE(SW:boolean):boolean
% Sets character folding status and returns old value. If character
% folding is true, then upper and lower case characters are folded
% into one character set, upper or lower depending on system. Which
% it is can be seen in SHPCASEUP!*.
(IFP (BOUNDP '!*!*LOW!-CASE)            % Test for lower case PSL
 (DE SETFOLDCASE (SW)
  (PROG1 !*RAISE (SETQ !*RAISE (COND (SW (INT2ID 20))))) ) )

% SYESP(MESSAGE:any):boolean
% If running in batch then SYESP always returns the value of
% YESPDEFAULT!* which initially is NIL.
% If running interactively then SYESP prints MESSAGE on the standard
% output, followed by "Answer Y or N: " and then reads a character
% from standard input.
% If that character is N then NIL is returned, if it is Y then T is
% returned, else "Answer Y or N: " is printed again and a new
% character is read. This goes on until Y or N is read but not more
% than ASKREPEAT!* (normally 5) times. After the last time with no Y
% or N read, YESPDEFAULT!* is returned.
(DE SYESP (MSG) (IYESP MSG T YESPDEFAULT!*))

% SNOP(MESSAGE:any):boolean
% If running in batch then SNOP always returns the value of NOPDEFAULT!*
% which initially is NIL.
% Similar to SYESP, except that Y gives NIL and N gives T.
(DE SNOP (MSG) (IYESP MSG NIL NOPDEFAULT!*))

%(DE IYESP (MSG YES DEFLT) (COND
%  ((NOT !*INT) DEFLT)
%  (T (SLET ((CT ASKREPEAT!*))
%    (STDOUT (QTRPRI) (PRIN2L MSG) (TERPRI))
%    (UNTIL (OR (EQ MSG !$EOF!$)
%               (EQ (SETQ MSG (CAR (EXPLODEC MSG))) 'Y) (EQ MSG 'N)
%               (IMINUSP (IDECR CT)) )
%           (SETQ MSG (FOLDCASE T
%             (STDIN (PROMPTER "Answer Y or N:" (PROMPT) (READ))) )))
%    (COND
%     ((EQ MSG 'Y) YES)
%     ((EQ MSG 'N) (NOT YES))
%     (T (WARNINGFT 48 "Using default %r" (COND
%          ((COND (YES (NOT DEFLT)) (T DEFLT)) 'N)
%          (T 'Y) ))
%        DEFLT) ) )) ))

(IFP (NOT !*RSHEEP) (SAMEDEF BYE EXITLISP))

% QUIT defined.

(FLAG '(CAPTURESFSTKS UNBINDSFSTKS)
      'LOSE)                            % Not needed
(FLAG '(AMEMQ AATSOC AEQN IEQN XAMEMQ XAATSOC XAEQN XIEQN
        ERRSETT  % ULCASE
        EXPLODEC ORDERP STDINP STDOUTP PRIN2D PRIN1D
        TTAB PGLINE PROMPT SYESP SNOP)  % IYESP
      'LOSE)                            % Defined in the present file.
(IFP
 (AND
  (NUMBERP (SYSTYPE 'LISPVERS))
  (GREATERP (SYSTYPE 'LISPVERS) 3.3) )
 (FLAG '(FILEOUT)
       'LOSE)           % Use exfn2i version for new PSL-version.
 (FLAG '(IOCHANS STDIN STDOUT STDINOUT FILEIN FILEOUT)
       'LOSE) )
% DELQIP defined differently.
(FLAG '(RPLACW COPY PNTH NTH INTERSECTIONQ ADJOINQ
        LASTPAIR SUBLA FLAG1 REMFLAG1
        EXPRP MACROP FEXPRP INTERNP LIST2VECTOR VECTOR VECTOR2LIST
        GETFNTYPE  % GETFNTYPE different. No problem?
        FILEP PRIN2T SPACES RESET)
      'LOSE)                            % Defined in Bare PSL.
(IFP (MEMQ 'USEFUL OPTIONS!*)           % Defined in PSL Module USEFUL.
 (FLAG '(INCR DECR POP XOR) 'LOSE) )
(IFP (MEMQ 'COMMON OPTIONS!*)           % Defined in PSL Module COMMON.
 (FLAG '(LISTP) 'LOSE) )
(IFP (AND (GETD 'EVENP) (CODEP (CDR (GETD 'EVENP))))    % Defined in
 (FLAG '(EVENP) 'LOSE) )                % PSL Module COMMON and in the
                                        % compiler for PSL version 3.4.
(IFP (BOUNDP '!*!*LOW!-CASE) (FLAG '(SETFOLDCASE) 'LOSE))
% GCMESSAGE defined with different meaning in some versions of PSL 4.2
(IFP (GETD 'GCMESSAGE) (FLAG '(GCMESSAGE) 'LOSE))

(INCLUDE "../exfn2i.sl")

(REMFLAG '(CAPTURESFSTKS UNBINDSFSTKS)
      'LOSE)                    % Not needed
(REMFLAG '(AMEMQ AATSOC AEQN IEQN XAMEMQ XAATSOC XAEQN XIEQN
        ERRSETT  % ULCASE
        EXPLODEC ORDERP STDINP STDOUTP PRIN2D PRIN1D
        TTAB PGLINE PROMPT SYESP SNOP)  % IYESP
      'LOSE)                            % Defined in the present file.
(IFP
 (AND
  (NUMBERP (SYSTYPE 'LISPVERS))
  (GREATERP (SYSTYPE 'LISPVERS) 3.3) )
 (REMFLAG '(FILEOUT)
       'LOSE)           % Use exfn2i version for new PSL-version.
 (REMFLAG '(IOCHANS STDIN STDOUT STDINOUT FILEIN FILEOUT)
       'LOSE) )
% DELQIP defined differently.
(REMFLAG '(RPLACW COPY PNTH NTH INTERSECTIONQ ADJOINQ
        LASTPAIR SUBLA FLAG1 REMFLAG1
        EXPRP MACROP FEXPRP INTERNP LIST2VECTOR VECTOR VECTOR2LIST
        GETFNTYPE  % GETFNTYPE different. No problem?
        FILEP PRIN2T SPACES RESET)
      'LOSE)                            % Defined in Bare PSL.
(IFP (MEMQ 'USEFUL OPTIONS!*)           % Defined in PSL Module USEFUL.
 (REMFLAG '(INCR DECR POP XOR) 'LOSE) )
(IFP (AND (GETD 'EVENP) (CODEP (CDR (GETD 'EVENP))))    % Defined in
 (REMFLAG '(EVENP) 'LOSE) )             % PSL Module COMMON and in the
                                        % compiler for PSL version 3.4.
(IFP (BOUNDP '!*!*LOW!-CASE) (REMFLAG '(SETFOLDCASE) 'LOSE))
% GCMESSAGE defined with different meaning in some versions of PSL 4.2
(IFP (GETD 'GCMESSAGE) (REMFLAG '(GCMESSAGE) 'LOSE))

% File names will be converted to lower case (also for VMS).
(SETCONSTANTS (!*OSCASEUP NIL))

(IFP (BOUNDP '!*!*LOW!-CASE)            % Test for lower case PSL
% PSL from version 4.2 (?) uses lower case internally.
 (SETCONSTANTS
  (!*ILISPCASEUP NIL)
  (!*SHPCASEUP NIL) )
% PSL up to version 3.4 (?) uses upper case internally.
 (SETCONSTANTS
  (!*ILISPCASEUP T)
  (!*SHPCASEUP T) ) )

(DE PSLTOPLEVEL ()      % PSL Style Lisp top loop. Not currently used.
 (SLET
  ((CURRENTREADMACROINDICATOR!* 'LISPREADMACRO)
   (CURRENTSCANTABLE!* LISPSCANTABLE!*) )
  (TOPLOOP 'READ 'PRINTSUPPRESS 'EVAL
   (MAKESTR (REVERSE (CDDR (REVERSE (EXPLODEC INITPROMPT!*)))))
   LISPBANNER!* )))

(DE PRINTSUPPRESS (PRNEVL!$X)
 (COND
  ((AND !*SUPPRESSANS (EQ PRNEVL!$X SUPPRESSANS!*)) NIL)
  (T (QTRPRI) (PRIN1 PRNEVL!$X) (TERPRI)) ))

% Note! Redefine MAIN, a top level function in PSL.
(DE MAIN () (FUNCALL TOPLEVEL!*))

(DE NEWLISPTOP ()
  (ERRSETT (CATCH 'RESET (XSLISPTOP)))
  (QTRPRI) (PRIN2 "Returning to top level") (TERPRI)
  (CLEARIO)
% (CLEARBINDINGS)               % Testing.
  (INITIALIZEINTERRUPTS)
  (IFP (EQ 'VMS (SYSTYPE 'OS)) (RELOADFOREIGNIMAGES))
% (EVALINITFORMS)
  (NEWLISPTOP)                  % The only way for uncompiled PSL?
   )                            % An UNTIL-construct will not do.

(DE EXTRAINIT NIL
 (PROGN
  (SETQ !*USERMODE NIL)
  (IFP (GETD 'GETENV)           % Unix case.
   (SLET
    ((HST (GETENV "HOST"))
     (HSTSPLIT (FILNAM HST))
     (DOMN (GETENV "DOMAINNAME")) )
    (COND
     ((OR (NULL DOMN) (EQUAL "(none)" DOMN))
      (SETQ DOMN (CDDR HSTSPLIT)) ))
    (COND
     ((AND HST (NOT (EQUAL "" HST)))
      (SETSYSTYPE 'NODE (CONCATIDL (EXPLODEC (CADR HSTSPLIT)))) ) )
    (COND
     ((AND DOMN (NOT (EQUAL "" DOMN)))
      (SETSYSTYPE 'DOMAIN DOMN) ) ) )
  (IFP (GETD 'MACHINENAME)     % VMS PSL 3.2A and later.
   (PROGN
    (SETSYSTYPE 'MACHINEVERS (HARDWAREVERSION))
    (COND
     ((EXPLODEC (MACHINENAME))
      (SETSYSTYPE 'NODE (CONCATIDL (EXPLODEC (MACHINENAME)))) )))))))

(DE EXPLODE3 (X) (REVERSIP (CDR (REVERSIP (CDR (EXPLODE X))))))

(IFP (EQ 'DOS386 (SYSTYPE 'OS)) % No way to access commandline yet.
  (SETQ !!COMMANDLINE NIL)
  (SETQ !!COMMANDLINE T))

(SETGLOBALS COMMANDLINE!* ARGV!* ARGV0!*)
(IFP (MEMQ (SYSTYPE 'MACHINE) '(ALPHA)) (SETFLUIDS ARGC))

(IFP
 (EQ 'VMS (SYSTYPE 'OS))
 (DE INITCOMMANDLINE ()                 % VMS version.
  (SETQ COMMANDLINE!* (GET!-COMMAND!-STRING))
  (SLET
   ((COMLST (EXPLODE3 COMMANDLINE!*)) NXTCH NXTCOML REVARGL)
   (WHILE
    COMLST
    (WHILE
     (NOT (OR (NULL COMLST) (EQ '!  (SETQ NXTCH (POP COMLST)))))
     (XPUSH NXTCOML NXTCH) )
    (XPUSH REVARGL (MAKESTR (REVERSE NXTCOML)))
    (SETQ NXTCOML NIL) )
   (PROG1 (SETQ ARGV!* (REVERSIP REVARGL))
    (SETQ ARGV0!* (CHKERARGV)) ) ))
 (DE INITCOMMANDLINE ()                 % Unix version.
  % A bugfix for Unix Alpha PSL:
  (IFP (MEMQ (SYSTYPE 'MACHINE) '(ALPHA)) (SETQ ARGC (MAX ARGC 15)))
  (GETUNIXARGS)                         % Needed only for some PSLs?
  (SETQ ARGV!*
   (SLET
    (A (I (ADD1 (UPBV UNIXARGS!*))))
    (WHILE (NOT (XZEROP (XDECR I)))
     (IFP
      (MEMQ (SYSTYPE 'MACHINE) '(ALPHA))
      (COND ((GETV UNIXARGS!* I) (XPUSH A (GETV UNIXARGS!* I))))
      (XPUSH A (GETV UNIXARGS!* I)) ) )
    A ))
  (SETQ ARGV0!* (GETSTARTUPNAME)) ) )

(DE NEXTONCOMLIN () (SLET ((AAAA ARGV!*))
   (COND (AAAA (SETQ ARGV!* (CDR ARGV!*))))
   AAAA ))

(DE SEXPRFROMCOM ()
 (SLET ((A (CHKERARGV)) B)
  (WHILE
   (AND (NOT (PAIRP (SETQ B (ERRSET (REREADSTRING A) NIL))))
    (SETQ B (CAR (NEXTONCOMLIN))) )
    (SETQ A (MAKESTR (NCONC (EXPLODE3 A) (CONS '!  (EXPLODE3 B))))) )
  (COND
   (B (CAR B))
   (T (REREADSTRING A)) ) ) )

% Switch -P is for PSL pipes, two arguments. Must be used for WIN32 but
% may exist also for other version  PSL. It just skips two args here.
(DC P INITLINEFN () (CHKERARGV) (CHKERARGV))  % PSL pipes, two args

(DE REREADSTRING (X) (IMPLODE (EXPLODEC X)))

(DE INITSEARCHLIS ()
 (SLET ((A (PREFIXOF IMGFILE!*)) B (ARGV0 (PREFIXOF (PREFIXOF ARGV0!*))))
  (COND
   (IMGFILE!*           % If image file was given as argument to -f
    (SETQ TOPDIR!*      % for a PSL with .img files always set TOPDIR!*
     (COND              % from this information.
      ((AND A (SETQ B (PREFIXOF A)))
       B)
      (A ".")
      (T "..") ) ) )
   ((OR                 % Save TOPDIR!* for the benefit of *.ini files
     (AND ARGV0         % but only if it is an absolute directory or
      (ABSDIRP (SETQ ARGV0 (EXPLODEC (ULCASE ARGV0 NIL)))) )
     (NULL TOPDIR!*) )  % not set. Convert to lower case.
    (SETQ TOPDIR!* (MAKESTR ARGV0)) ) ) ) )
% Else start from old value of TOPDIR!* or current directory.

% To enable PSL to load image, used here to set TOPDIR!* and
% MSEARCHLIS!*.
(DC F INITLINEFN () (SETQ IMGFILE!* (CHKERARGV)))

(DC TD INITLINEFN () (CHKERARGV))       % PSL total size. For *.img

(FLAG
 '(NEWLISPTOP EXTRAINIT INITCOMMANDLINE
   NEXTONCOMLIN SEXPRFROMCOM INITSEARCHLIS)
 'LOSE)  % These have been defined in the present file.

(INCLUDE "../slini.sl")

(REMFLAG
 '(NEWLISPTOP EXTRAINIT INITCOMMANDLINE
   NEXTONCOMLIN SEXPRFROMCOM INITSEARCHLIS)
 'LOSE)  % These have been defined in the present file.

% All changes in INITLINEFNs caused by the PSL-implementation are now
% in the machine independent code.

(SAMEDEF DATETIME DATE)

% X!& and its value will be left as garbage in the saved image.
(DFEXPR SAVERUN (X!&)
 (SETQ !*ERRMSG T)
 (SETQ !*IPRN T)
 (FOREACH Y IN SFSTKS!* DO (STNIL Y))
 (FOREACH Y IN SAVEDO!* DO (COND ((PAIRP Y) (EVAL Y)) (T (STNIL Y))))
 (SETQ MODULNAM!* 'UNKNOWN)
 (SETQ LOADMSGTAB!* 0)
% (SET!-TOP!-LEVEL 'PSLTOPLEVEL)
 (SETQ X!& (COND ((PAIRP X!&) (CAR X!&)) (T X!&)))
 (COND
  ((AND (BOUNDP 'NOSAVE) NOSAVE)
   (PRIN2T "Ready to be saved") )
  (T (PRIN2T "About to save")
     (SAVESYSTEM
      INITMESSAGE!*
      (OUTFILENAME NIL X!& (IFP (EQ 'VMS (SYSTYPE 'OS)) "exe" NIL))
       '(
         (SLISPINIT)                    % For top loop change.
%        (ERRSETT (SLSPINI))            % For PSL style top loop.
         ) )
     (BYE) ) ) )

%### EXFN2.SL end
