%### SLINI.SL =======================================================
%
%       SLINI.SL
%       --------
%
%       Copyright (c) 1982 by Inge Frick, Institute of Theoretical
%       Physics, Vanadisvagen 9, S-113 46 Stockholm, Sweden.
%
%
%       This file contains definitions of a top level and some
%       initialisation functions.
%
%       If the variable !!COMMANDLINE is true then code will be
%       included that interprets the command line that starts up
%       Sheep lisp. In that case some the following system dependent
%       functions are needed: NEXTONCOMLIN, SEXPRFROMCOM and
%       INITCOMMANDLINE.
%
%====================================================================

(FUNCTIONS INITSEARCHLIS SET!-TOP!-LEVEL SLISPTOP XSLISPTOP ISLISPTOP
           SLISPINIT TRYINICHSWITCH CALINISWITCH CHKERARGV)

(SETGLOBALS                     % These globals are used by SLISPINIT
 (DATE!*        "7-NOV-1897")   % This date must be set appropriately.
 (SLISPTOP!*    SLISPTOP)   % Top level. A function with no arguments.
 (SYSINITFILE!* "SSLISP.INI")   % Name of system initialisation file.
 (INITFILE!*    "SLISP.INI")    % Name of initialisation file.
 (INITMESSAGE!* "Sheep Lisp ")  % Initial message.
 (INITPROMPT!*  "SL> ")
 TOPLEVEL!*
 TOPDIR!*
 LISPLIBDIR!*
 HOMEDIR!*
 CALLINITSW!*
 INITS!*                        % List of forms to evaluate initially.
 GOFN!* )                       % Function for -G and -H switches.

(SETFLUIDS !*TTYECHO MSEARCHLIS!* !*GOTOP !*STOPCMPARS !*INTFORCE
 !*QBATCH !*LOADMSG)

% No command line feature by default.
(NOTLOADTIME (QSET '!!COMMANDLINE NIL))

% In a system where one can change top level, SET!-TOP!-LEVEL should
% do that. The argument is the name of the new top level function.
(DS SET!-TOP!-LEVEL (X) (SETQ TOPLEVEL!* X))

(DE SLISPINIT () (ERRSETT (SLSPINI)) (NEWLISPTOP))

%This definition of NEWLISPTOP is intended for systems where one
% cannot change top level. NEWLISPTOP will execute the value of
% TOPLEVEL!* as a new top level.
(DE NEWLISPTOP ()
 (UNTIL NIL
  (ERRSETT (XSLISPTOP))
  (QTRPRI) (PRIN2 "Returning to top level") (TERPRI) ) )

(DE XSLISPTOP () (UNTIL NIL (FUNCALL TOPLEVEL!*)))

(DE SLISPTOP ()
 (QTRPRI)
 (TERPRI)
 (PROMPT)
 (SLET ((S!- (SLET ((!*ECHO !*TTYECHO)) (ERRSETT (READ)))))
  (COND
   ((AND (PAIRP S!-) (NOT (EQ (SETQ S!- (CAR S!-)) !$EOF!$)))
    (DFEVAL S!- T) )
   ((NOT (EQ S!- !$EOF!$)) NIL)
   (!*INT (TERPRI) (BELL))
   (T (BYE)) ) ) )

(NOTLOADTIME (DS INITSEARCHLIS () NIL))

(NOTLOADTIME (DS SETBATCH (QBT) (NOT QBT)))

(DE SLSPINI ()
 (PROG (ICOM!$ !*GOTOP !*STOPCMPARS !*QBATCH !*INTFORCE)
  (RDS NIL)
  (WRS NIL)
  (SET!-TOP!-LEVEL SLISPTOP!*)
  (SETPROMPTER INITPROMPT!*)
  (SETQ FIL!* NIL)
  (EXTRAINIT)           % Implementation specific initialisation.
  (IFP !!COMMANDLINE (PROGN
    (INITCOMMANDLINE)
    (WHILE (AND (NOT !*STOPCMPARS) (SETQ ICOM!$ (NEXTONCOMLIN)))
     (DOINICHCMD (CAR ICOM!$)) )
    (COND (!*GOTOP (GO E))) ))
  (COND ((NOT !*INTFORCE) (SETQ !*INT (SETBATCH !*QBATCH))))
  (DOINITFILE SYSINITFILE!* INITFILE!*)
  (IFP !!COMMANDLINE
   (COND (CALLINITSW!* (EVAL (CONS 'PROGN (REVERSIP CALLINITSW!*))))) )
  (COND (INITMESSAGE!* (PRIN2 INITMESSAGE!*)))
  (COND (DATE!* (PRIN2 "(") (PRIN2 DATE!*) (PRIN2 ")")))
E (QTRPRI)
  (IFP !!COMMANDLINE (SETQ CALLINITSW!* NIL)) ))

% This version of EXTRAINIT is a noop that can be used if there is
% no need for implementation specific initialisation. JEA.
(DE EXTRAINIT () NIL)

(DE DOINITFILE (SIFIL IFIL)
 (COND (INITS!* (EVAL (CONS 'PROGN INITS!*)) (SETQ INITS!* NIL)))
 (INITSEARCHLIS)
 (COND (SIFIL (LOADP SIFIL
                     (COND (TOPDIR!* (LIST NIL TOPDIR!*)) (T '(NIL)))
                     '(NIL) T NIL )))
 (QTRPRI)
 (COND ((AND IFIL (LOADP IFIL
          (CONS
           (CAR MSEARCHLIS!*)
           (APPEND HOMEDIR!* (CDR MSEARCHLIS!*)) )
          '(NIL) T NIL ))
        (TERPRI) )) )

% Include code that interprets the command line
(IFP !!COMMANDLINE (CPROGN

  (DE DOINICHCMD (ICOM!$) (COND
    ((TRYINICHSWITCH ICOM!$) NIL)
    ((FNP ICOM!$) (FUNCALL ICOM!$))
    (T (WARNINGFT 60
        "Unknown command line command %p ignored" ICOM!$) ) ))

  % This code contains changes by JEA
  (DE TRYINICHSWITCH (TRI!$X)
   (COND
    ((NOT (EQCAR (SETQ TRI!$X (EXPLODEC TRI!$X)) '!-) NIL))
    ((OR (TESTINITLINEFN (MAKEID (CDR TRI!$X) T) NIL)
      (FOREACH TRI!$Y IN (CDR TRI!$X) DO (TESTINITLINEFN TRI!$Y T)) )
     T ) ) )

  % Test argument as it is and with default case for INITLINEFN
  (DE TESTINITLINEFN (TRY!$Z WSW!$)
   (SLET (TRY!$F)
    (COND
     ((SETQ TRY!$F
       (OR
        (GET TRY!$Z 'INITLINEFN)
        (GET (ULCASE TRY!$Z !*SHPCASEUP) 'INITLINEFN) ) )
      (SETQ TRY!$F (FUNCALL TRY!$F))
      (XPUSH CALLINITSW!* TRY!$F) )
     (WSW!$
      (WARNINGFT 62 "Undefined init switch %p ignored" TRY!$Z) ))))

  % Defines the command line switch -L with one argument,
  % the name of a module to load.
  % In earlier implementations called -F, renamed to avoid clash for
  % PSLs with images. Originally called -f.
  (DPUT L INITLINEFN L!.INITLINEFN)
  (DE L!.INITLINEFN () (LIST 'MLOAD (MKQUOTE (CHKERARGV))))

  % Switch -E, evaluates s-expression and exits.
  (DC E INITLINEFN ()
   (SETQ !*QBATCH T)
   (LIST 'PROGN (C!.INITLINEFN) '(BYE)) )

  % Switch -C, evaluates s-expression that follows the switch
  % on the command line. It also stops parsing of command line.
  % Earlier called -e.
  (DPUT C INITLINEFN C!.INITLINEFN)
  (DE C!.INITLINEFN ()
   (SETQ !*STOPCMPARS T)
   (LIST 'ERRSETT (LIST 'DFEVAL (MKQUOTE (SEXPRFROMCOM)) NIL)) )

  (DC G INITLINEFN ()
   (SETQ !*QBATCH T)
   (LIST 'PROGN (H!.INITLINEFN) '(BYE)) )

  % Defines the command line switch -H with one argument, the name of a
  % module to load and execute. It also stops parsing of command line.
  % Earlier called -g.
  (DPUT H INITLINEFN H!.INITLINEFN)
  (DE H!.INITLINEFN ()
   (SETQ !*STOPCMPARS T)
   (LIST 'ERRSETT (L!.INITLINEFN) '(COND (GOFN!* (FUNCALL GOFN!*)))) )

  % Force interactive even if running in batch.
  % Earlier called -B.
  (DC J INITLINEFN () (SETQ !*INTFORCE T) (SETQ !*INT T) NIL)

  % Force not interactive even if not running in batch.
  % Earlier called -b.
  (DC B INITLINEFN () (SETQ !*INTFORCE T) (SETQ !*INT NIL))

  % To get messages when loading files. Earlier called -m.
  (DC M INITLINEFN () (SETQ !*LOADMSG T) NIL)

  % To avoid getting messages when loading files. Earlier called -M.
  (DC N INITLINEFN () (SETQ !*LOADMSG NIL) NIL)

  % Prevent loading of ini-files etc. Earlier called -x.
  (DC X INITLINEFN () (SETQ !*GOTOP T) NIL)

  (DE CHKERARGV ()
   (SLET ((X (NEXTONCOMLIN)))
    (COND ((NULL X) (ERRORFT 435 "Too few arguments on command line")))
    (CAR X) ) )

% (DE NEXTONCOMLIN () (MUSTDEFINE NEXTONCOMLIN "SLINI.SL"))

% (DE SEXPRFROMCOM () (MUSTDEFINE SEXPRFROMCOM "SLINI.SL"))

% (DE INITCOMMANDLINE () (MUSTDEFINE INITCOMMANDLINE "SLINI.SL"))
))                                      %End of IFP !!COMMANDLINE

%### SLINI.SL end
